from PyQt6.QtCore import Qt, pyqtSignal, QLine, QPoint, PYQT_VERSION_STR, QT_VERSION_STR
from PyQt6.QtGui import QPixmap, QPainter, QPen
from PyQt6.QtWidgets import (QPushButton, QDialog, QLabel, QFormLayout,
                             QDialogButtonBox, QVBoxLayout)

from lanzou.debug import SRC_DIR
from lanzou.gui.qss import others_style, btn_style


class AboutDialog(QDialog):
    check_update = pyqtSignal(str, bool)

    def __init__(self, parent=None):
        super(AboutDialog, self).__init__(parent)
        self._ver = ''
        self._github = 'https://github.com/Leon406/lanzou-gui'
        self._api_url = 'https://github.com/zaxtyson/LanZouCloud-API'
        self._gitee = 'https://gitee.com/LeonShih/lanzou-gui'
        self._home_page = 'https://github.com/Leon406/lanzou-gui'
        self.initUI()
        self.setStyleSheet(others_style)

    def set_values(self, version):
        self._ver = version
        self.lb_name_text.setText(f"v{version}  (点击检查更新)")  # 更新版本

    def show_update(self, ver, msg):
        self.lb_new_ver = QLabel("新版")  # 检测新版
        self.lb_new_ver_msg = QLabel()
        self.lb_new_ver_msg.setOpenExternalLinks(True)
        self.lb_new_ver_msg.setWordWrap(True)
        if ver != '0':
            self.lb_name_text.setText(f"{self._ver}  ➡  {ver}")
        self.lb_new_ver_msg.setText(msg)
        self.lb_new_ver_msg.setMinimumWidth(700)
        if self.form.rowCount() < 5:
            self.form.insertRow(1, self.lb_new_ver, self.lb_new_ver_msg)

    def initUI(self):
        self.setWindowTitle("关于 lanzou-gui")
        about = f'本项目使用PyQt6实现图形界面，可以完成蓝奏云的大部分功能<br/> \
    得益于 <a href="{self._api_url}">API</a> 的功能，可以间接突破单文件最大 100MB 的限制，同时增加了批量上传/下载的功能<br/> \
Python 依赖见<a href="{self._github}/blob/master/requirements.txt">requirements.txt</a>，\
<a href="{self._github}/releases">releases</a> 有打包好了的 Windows 可执行程序，但可能不是最新的'
        project_url = f'<a href="{self._home_page}">主页</a> | <a href="{self._github}">repo</a> | \
                        <a href="{self._gitee}">mirror repo</a>'
        self.logo = QLabel()  # logo
        self.logo.setPixmap(QPixmap(SRC_DIR + "logo2.gif"))
        self.logo.setStyleSheet("background-color:rgb(255,255,255);")
        self.logo.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.lb_qt_ver = QLabel("依赖")  # QT 版本
        self.lb_qt_text = QLabel(f"QT: {QT_VERSION_STR}, PyQt: {PYQT_VERSION_STR}")  # QT 版本
        self.lb_name = QLabel("版本")  # 版本
        self.lb_name_text = QPushButton("")  # 版本
        self.lb_name_text.setToolTip("点击检查更新")
        ver_style = "QPushButton {border:none; background:transparent;font-weight:bold;color:blue;}"
        self.lb_name_text.setStyleSheet(ver_style)
        self.lb_name_text.clicked.connect(lambda: self.check_update.emit(self._ver, True))
        self.lb_about = QLabel("关于")  # about
        self.lb_about_text = QLabel()
        self.lb_about_text.setText(about)
        self.lb_about_text.setOpenExternalLinks(True)
        self.lb_author = QLabel("作者")  # author
        self.lb_author_mail = QLabel(" <a href='mailto:deadogone@gmail.com'>leon406</a>, "
                                     "<a href='mailto:rachpt@126.com'>rachpt</a>"
                                     )
        self.lb_author_mail.setOpenExternalLinks(True)
        self.lb_update = QLabel("项目")  # 更新
        self.lb_update_url = QLabel(project_url)
        self.lb_update_url.setOpenExternalLinks(True)
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Close)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Close).setText("关闭")
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.setStyleSheet(btn_style)

        self.line = QLine(QPoint(), QPoint(550, 0))
        self.lb_line = QLabel('<html><hr /></html>')

        vbox = QVBoxLayout()
        vbox.addWidget(self.logo)
        vbox.addStretch(1)
        self.form = QFormLayout()
        self.form.setLabelAlignment(Qt.AlignmentFlag.AlignRight)
        self.form.setFormAlignment(Qt.AlignmentFlag.AlignLeft)
        self.form.setHorizontalSpacing(40)
        self.form.setVerticalSpacing(15)
        self.form.addRow(self.lb_qt_ver, self.lb_qt_text)
        self.form.addRow(self.lb_name, self.lb_name_text)
        self.form.addRow(self.lb_update, self.lb_update_url)
        self.form.addRow(self.lb_author, self.lb_author_mail)
        self.form.addRow(self.lb_about, self.lb_about_text)
        self.form.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)  # 覆盖MacOS的默认样式
        vbox.addLayout(self.form)
        vbox.addStretch(1)
        vbox.addWidget(self.lb_line)

        self.setLayout(vbox)
        self.setMinimumWidth(720)

    def paintEvent(self, event):
        QDialog.paintEvent(self, event)
        if not self.line.isNull():
            painter = QPainter(self)
            pen = QPen(Qt.GlobalColor.red, 3)
            painter.setPen(pen)
            painter.drawLine(self.line)



from lanzou.api.core import LanZouCloud

version = '2.6.8'


def why_error(code):
    """错误原因"""
    if code == LanZouCloud.URL_INVALID:
        return '分享链接无效'
    elif code == LanZouCloud.LACK_PASSWORD:
        return '缺少提取码'
    elif code == LanZouCloud.PASSWORD_ERROR:
        return '提取码错误'
    elif code == LanZouCloud.FILE_CANCELLED:
        return '分享链接已失效'
    elif code == LanZouCloud.ZIP_ERROR:
        return '解压过程异常'
    elif code == LanZouCloud.NETWORK_ERROR:
        return '网络连接异常'
    elif code == LanZouCloud.CAPTCHA_ERROR:
        return '验证码错误'
    else:
        return f'未知错误 {code}'


__all__ = ['utils', 'types', 'models', 'LanZouCloud', 'version', 'why_error']



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import QDialog, QLabel, QGridLayout, QDialogButtonBox, QComboBox

from lanzou.debug import SRC_DIR
from lanzou.gui.others import AutoResizingTextEdit
from lanzou.gui.qss import dialog_qss_style


class MoveFileDialog(QDialog):
    """移动文件对话框"""
    new_infos = pyqtSignal(object)

    def __init__(self, parent=None):
        super(MoveFileDialog, self).__init__(parent)
        self.infos = None
        self.dirs = {}
        self.initUI()
        self.setStyleSheet(dialog_qss_style)

    def update_ui(self):
        names = "\n".join([i.name for i in self.infos])
        names_tip = "\n".join([i.name for i in self.infos])
        self.tx_name.setText(names)
        self.tx_name.setToolTip(names_tip)

        self.tx_new_path.clear()
        f_icon = QIcon(SRC_DIR + "folder.gif")
        for f_name, fid in self.dirs.items():
            if len(f_name) > 50:  # 防止文件夹名字过长？
                f_name = f_name[:47] + "..."
            self.tx_new_path.addItem(f_icon, "id：{:>8}，name：{}".format(fid, f_name))

    def set_values(self, infos, all_dirs_dict):
        self.infos = infos
        self.dirs = all_dirs_dict
        self.update_ui()
        self.exec()

    def initUI(self):
        self.setWindowTitle("移动文件(夹)")
        self.setWindowIcon(QIcon(SRC_DIR + "move.png"))
        self.lb_name = QLabel()
        self.lb_name.setText("文件(夹)名：")
        self.lb_name.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)
        self.tx_name = AutoResizingTextEdit()
        self.tx_name.setFocusPolicy(Qt.FocusPolicy.NoFocus)  # 只读
        self.tx_name.setReadOnly(True)
        self.lb_new_path = QLabel()
        self.lb_new_path.setText("目标文件夹：")
        self.lb_new_path.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)
        self.tx_new_path = QComboBox()

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")

        self.grid = QGridLayout()
        self.grid.setSpacing(10)
        self.grid.addWidget(self.lb_name, 1, 0)
        self.grid.addWidget(self.tx_name, 1, 1)
        self.grid.addWidget(self.lb_new_path, 2, 0)
        self.grid.addWidget(self.tx_new_path, 2, 1)
        self.grid.addWidget(self.buttonBox, 3, 0, 1, 2)
        self.setLayout(self.grid)
        self.buttonBox.accepted.connect(self.btn_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        self.setMinimumWidth(280)

    def btn_ok(self):
        new_id = self.tx_new_path.currentText().split("，")[0].split("：")[1]
        for index in range(len(self.infos)):
            self.infos[index].new_id = int(new_id)
        self.new_infos.emit(self.infos)



from time import sleep

from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger


class GetAllFoldersWorker(QThread):
    '''获取所有文件夹name与fid，用于文件移动'''
    infos = pyqtSignal(object, object)
    msg = pyqtSignal(str, int)
    moved = pyqtSignal(bool, bool, bool)

    def __init__(self, parent=None):
        super(GetAllFoldersWorker, self).__init__(parent)
        self._disk = None
        self.org_infos = None
        self._mutex = QMutex()
        self._is_work = False
        self.move_infos = None

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, org_infos):
        self.org_infos = org_infos  # 对话框标识文件与文件夹
        self.move_infos = []  # 清除上次影响
        self.start()

    def move_file(self, infos):
        '''移动文件至新的文件夹'''
        self.move_infos = infos  # file_id, folder_id, f_name, type(size)
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def move_file_folder(self, info, no_err: bool, r_files: bool, r_folders: bool):
        """移动文件(夹)"""
        # no_err 判断是否需要更新 UI
        if info.is_file:  # 文件
            if self._disk.move_file(info.id, info.new_id) == LanZouCloud.SUCCESS:
                self.msg.emit(f"{info.name} 移动成功！", 3000)
                no_err = True
                r_files = True
            else:
                self.msg.emit(f"移动文件{info.name}失败！", 4000)
        else:  # 文件夹
            if self._disk.move_folder(info.id, info.new_id) == LanZouCloud.SUCCESS:
                self.msg.emit(f"{info.name} 移动成功！", 3000)
                no_err = True
                r_folders = True
            else:
                self.msg.emit(f"移动文件夹 {info.name} 失败！移动的文件夹中不能包含子文件夹！", 4000)
        return no_err, r_files, r_folders

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            if self.move_infos:  # 移动文件
                no_err = False
                r_files = False
                r_folders = False
                for info in self.move_infos:
                    try:
                        no_err, r_files, r_folders = self.move_file_folder(info, no_err, r_files, r_folders)
                    except TimeoutError:
                        self.msg.emit(f"移动文件(夹) {info.name} 失败，网络超时！请稍后重试", 5000)
                    except Exception as e:
                        logger.error(f"GetAllFoldersWorker error: e={e}")
                        self.msg.emit(f"移动文件(夹) {info.name} 失败，未知错误！", 5000)
                if no_err:  # 没有错误就更新ui
                    sleep(2.1)  # 等一段时间后才更新文件列表
                    self.moved.emit(r_files, r_folders, False)
            else:  # 获取所有文件夹
                try:
                    self.msg.emit("网络请求中，请稍候……", 0)
                    all_dirs_dict = self._disk.get_move_folders().name_id
                    self.infos.emit(self.org_infos, all_dirs_dict)
                    self.msg.emit("", 0)  # 删除提示信息
                except TimeoutError:
                    self.msg.emit("网络超时！稍后重试", 6000)
                except Exception as e:
                    logger.error(f"GetAllFoldersWorker error: e={e}")
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



import os
import re
import sys
from logging import getLevelName, DEBUG, ERROR

from PyQt6.QtCore import Qt, QCoreApplication, QTimer, QUrl, QSize
from PyQt6.QtGui import QIcon, QStandardItemModel, QDesktopServices, QKeySequence, QAction, QShortcut
from PyQt6.QtWidgets import (QApplication, QAbstractItemView, QHeaderView, QMenu, QStyle,
                             QPushButton, QFileDialog, QMessageBox, QSystemTrayIcon)

from lanzou.api import LanZouCloud
from lanzou.api.models import FolderList
from lanzou.api.types import RecFolder, FolderDetail, ShareItem
from lanzou.api.utils import convert_file_size_to_int as format_size_int
from lanzou.api.utils import time_format, time_stamp
from lanzou.debug import logger, USER_HOME, SRC_DIR
from lanzou.gui import version
from lanzou.gui.config import config
from lanzou.gui.dialogs import *
from lanzou.gui.models import DlJob, Tasks, FileInfos, FolderInfos, ShareFileInfos
from lanzou.gui.others import MyStandardItem as QStandardItem
from lanzou.gui.others import set_file_icon, TableDelegate
from lanzou.gui.qss import *
from lanzou.gui.ui import Ui_MainWindow
from lanzou.gui.workers import *
from lanzou.gui.workers.manager import change_size_unit

__ALL__ = ['MainWindow']


def get_logo_path():
    """返回 logo 路径"""
    return SRC_DIR + 'lanzou_logo2.png'


def get_lanzou_logo():
    return QIcon(get_logo_path())


def open_in_explorer(task):
    QDesktopServices.openUrl(QUrl.fromLocalFile(task.path))


def open_file(task):
    new_name = re.sub(r"(\.\w+)\.enc", r"\1", task.name)
    url = QUrl.fromLocalFile(task.path + "/" + new_name)
    if not QDesktopServices.openUrl(url):
        logger.error(f"open file failed{url}", )


def time_table_item(data):
    try:
        item = QStandardItem(data.time)
        item.setData(time_stamp(data.time), Qt.ItemDataRole.UserRole)
    except Exception as e:
        print("errr", e)
    return item


class MainWindow(Ui_MainWindow):

    def __init__(self):
        super(MainWindow, self).__init__()
        self.setupUi(self)
        self.init_variables()
        self.init_workers()
        self.update_lanzoucloud_settings()

        self.main_menu_add_slot()
        self.init_extract_share_ui()
        self.init_disk_ui()
        self.init_rec_ui()
        self.init_jobs_ui()
        self.create_left_menus()

        self.setWindowTitle("蓝奏云客户端 - {}".format(version))
        self.setStyleSheet(qssStyle)
        if self.upgrade:  # 检测新版
            self.check_update_worker.set_values(version, False)
        self.clipboard_monitor()  # 系统粘贴板
        # self.call_login_launcher()

    def create_tray(self):
        """创建 系统托盘"""
        if not self._created_tray:
            self.tray = QSystemTrayIcon(get_lanzou_logo(), parent=self)
            show_action = QAction("显示窗口", self)
            hide_action = QAction("最小化到托盘", self)
            quit_action = QAction("退出程序", self)
            show_action.triggered.connect(self.show)
            hide_action.triggered.connect(self.hide)
            quit_action.triggered.connect(self.Exit)
            show_action.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_ComputerIcon))
            hide_action.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_TitleBarMinButton))
            quit_action.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_TitleBarCloseButton))
            self.tray.activated[QSystemTrayIcon.ActivationReason].connect(self.icon_activated)  # 托盘点击事件
            tray_menu = QMenu()
            # tray_menu = QMenu(QApplication.desktop())
            tray_menu.addAction(show_action)
            tray_menu.addAction(hide_action)
            tray_menu.addAction(quit_action)
            self.tray.setContextMenu(tray_menu)
            tip_title = f"蓝奏云客户端 <{self._user}>" if self._user else "蓝奏云客户端"
            self.tray.setToolTip(f"{tip_title}\n双击到托盘")
            self.tray.show()
            self._created_tray = True

    def icon_activated(self, reason):
        if reason == QSystemTrayIcon.ActivationReason.DoubleClick:
            tip_title = f"蓝奏云客户端 <{self._user}>" if self._user else "蓝奏云客户端"
            if self.isHidden():
                self.show()
                self.tray.setToolTip(f"{tip_title}\n双击到托盘")
            else:
                self.hide()
                self.tray.setToolTip(f"{tip_title}\n双击显示")

    def main_menu_add_slot(self):
        """主菜单添加槽函数"""
        self.login.triggered.connect(self.show_login_dialog)  # 登录
        self.logout.triggered.connect(self.call_logout)  # 登出
        self.download.triggered.connect(self.call_download_shortcut)  # 快捷键下载
        self.delete.triggered.connect(self.call_delete_shortcut)  # 快捷键删除
        self.upload.triggered.connect(self.show_upload_dialog_menus)
        self.setting_menu.triggered.connect(lambda: self.setting_dialog.open_dialog(self._config))
        self.show_toolbar.triggered.connect(self.show_toolbar_slot)
        self.how.triggered.connect(self.open_wiki_url)
        self.about.triggered.connect(self.about_dialog.exec)
        self.tabWidget.currentChanged.connect(self.call_change_tab)  # tab 切换时更新
        self.merge_file.triggered.connect(self.merge_file_dialog.exec)  # 合并文件

    def init_variables(self):
        self._disk = LanZouCloud()
        self._config = config
        self._user = None  # 当前登录用户名
        self._folder_list = {}  # disk 工作目录文件夹
        self._file_list = {}  # disk 工作目录文件
        self._extract_folder_list = {}  # share 提取子文件夹
        self._extract_show_dir = []  # share 提取文件显示子文件夹
        self._extract_setted_head = False  # share 提取界面设置表头标示
        self._extract_count = 0  # share 提取界面文件数
        self._path_list = FolderList()
        self._path_list_old = FolderList()
        self._locs = {}
        self._parent_id = -1  # --> ..
        self._work_name = ""  # share disk rec, not use now
        self._work_id = -1  # disk folder id
        self._old_work_id = self._work_id  # 用于上传完成后判断是否需要更新disk界面
        self._show_to_tray_msg = True
        self._created_tray = False
        self._tasks = Tasks()
        self._dl_jobs_lists = {}
        self._up_jobs_lists = {}
        self._to_tray = False
        self._show_subfolder = False  # 提取界面递归展示子文件

    def set_disk(self):
        """方便切换用户更新信息"""
        self.task_manager.set_disk(self._disk)
        self.get_shared_info_thread.set_disk(self._disk)
        self.login_launcher.set_disk(self._disk)
        self.list_refresher.set_disk(self._disk)
        self.remove_files_worker.set_disk(self._disk)
        self.get_rec_lists_worker.set_disk(self._disk)
        self.rec_manipulator.set_disk(self._disk)
        self.desc_pwd_fetcher.set_disk(self._disk)
        self.logout_worker.set_disk(self._disk)
        self.rename_mkdir_worker.set_disk(self._disk)
        self.set_pwd_worker.set_disk(self._disk)
        self.more_info_worker.set_disk(self._disk)
        self.all_folders_worker.set_disk(self._disk)

    def update_lanzoucloud_settings(self):
        """更新LanzouCloud实例设置"""
        self._user = self._config.name
        settings = self._config.settings
        self._disk.set_timeout(settings["timeout"])
        self._disk.set_max_size(settings["max_size"])
        self.task_manager.set_thread(settings["download_threads"])  # 同时下载任务数量
        self.share_set_dl_path.setText(self._config.path)  # 提取界面下载路径
        self.time_fmt = settings["time_fmt"]  # 时间显示格式
        self._to_tray = settings["to_tray"]
        self.watch_clipboard = settings["watch_clipboard"]
        self.upgrade = settings["upgrade"] if 'upgrade' in settings else True  # 自动检测更新
        self.upload_dialog.set_pwd_desc_bigfile(settings)
        self.task_manager.set_allow_big_file(settings["allow_big_file"])
        if 'upload_delay' in settings:
            delay = int(settings["upload_delay"])
            if delay > 0:
                self._disk.set_upload_delay((delay / 2, delay))
            else:
                self._disk.set_upload_delay((0, 0))
        # debug
        debug = settings["debug"]
        if debug:
            if getLevelName(logger.level) != "DEBUG":
                logger.setLevel(DEBUG)
                logger.debug("\n" + "=" * 69)
                logger.debug(f"Start New Debug: version {version}")
        else:
            logger.setLevel(ERROR)
        # 托盘图标
        if self._to_tray:
            self.create_tray()
        elif self._created_tray:
            self.tray.hide()
            del self.tray
            self._created_tray = False

    def init_workers(self):
        # 登录器
        self.login_launcher = LoginLuncher()
        self.login_launcher.update_username.connect(self._config.set_username)
        self.login_launcher.update_cookie.connect(self._config.set_cookie)  # 目前失效
        self.login_launcher.code.connect(self.login_update_ui)
        # 登出器
        self.logout_worker = LogoutWorker()
        self.logout_worker.succeeded.connect(self.call_logout_update_ui)
        # 任务管理器
        self.task_manager = TaskManager()
        self.task_manager.mgr_msg.connect(self.show_status)
        self.task_manager.update.connect(self.update_jobs_info)
        self.task_manager.mgr_finished.connect(self.call_show_mgr_finished)
        # 获取更多信息，直链、下载次数等
        self.info_dialog = InfoDialog()  # 对话框
        self.info_dialog.setWindowModality(Qt.WindowModality.ApplicationModal)  # 窗口前置
        self.more_info_worker = GetMoreInfoWorker()  # 后台更新线程
        self.more_info_worker.msg.connect(self.show_status)
        self.more_info_worker.infos.connect(lambda: self.pause_extract_clipboard(True))  # 禁用剪切板监听
        self.more_info_worker.infos.connect(self.info_dialog.set_values)
        self.more_info_worker.share_url.connect(self.call_copy_share_url)
        self.more_info_worker.dl_link.connect(self.info_dialog.set_dl_link_tx)
        self.info_dialog.get_dl_link.connect(self.more_info_worker.get_dl_link)
        self.info_dialog.closed.connect(lambda: self.pause_extract_clipboard(False))  # 恢复剪切板监听
        # 登录文件列表更新器
        self.list_refresher = ListRefresher()
        self.list_refresher.err_msg.connect(self.show_status)
        self.list_refresher.infos.connect(self.update_disk_lists)
        self.list_refresher.infos.connect(lambda: self.show_status(""))

        # 显示移动文件对话框
        self.move_file_dialog = MoveFileDialog()
        # 获取所有文件夹fid，并移动
        self.all_folders_worker = GetAllFoldersWorker()
        self.all_folders_worker.msg.connect(self.show_status)
        self.move_file_dialog.new_infos.connect(self.all_folders_worker.move_file)  # 调用移动线程
        self.all_folders_worker.infos.connect(self.move_file_dialog.set_values)
        self.all_folders_worker.moved.connect(self.on_moved)  # 更新文件列表

        # 重命名、修改简介、新建文件夹
        self.rename_mkdir_worker = RenameMkdirWorker()
        self.rename_mkdir_worker.msg.connect(self.show_status)
        self.rename_mkdir_worker.update.connect(self.list_refresher.set_values)  # 更新界面
        # 设置文件(夹)提取码
        self.set_pwd_worker = SetPwdWorker()
        self.set_pwd_worker.msg.connect(self.show_status)
        self.set_pwd_worker.update.connect(self.list_refresher.set_values)  # 更新界面
        # 删除文件(夹)
        self.remove_files_worker = RemoveFilesWorker()
        self.remove_files_worker.msg.connect(self.show_status)  # 显示错误提示
        self.remove_files_worker.finished.connect(lambda: self.list_refresher.set_values(self._work_id))  # 更新界面
        # 上传器，信号在登录更新界面设置
        self.upload_dialog = UploadDialog(USER_HOME)
        self.upload_dialog.new_infos.connect(self.call_upload)
        # 文件描述与提取码更新器
        self.desc_pwd_fetcher = DescPwdFetcher()
        self.desc_pwd_fetcher.desc.connect(self.call_update_desc_pwd)
        self.desc_pwd_fetcher.tasks.connect(self.call_task_manager_thread)  # 连接下载管理器线程

        # 重命名、修改简介与新建文件夹对话框
        self.rename_dialog = RenameDialog()
        self.rename_dialog.out.connect(self.call_rename_mkdir_worker)
        # 修改设置 提取码对话框
        self.set_pwd_dialog = SetPwdDialog()
        self.set_pwd_dialog.new_infos.connect(self.set_passwd)
        # 菜单栏关于
        self.about_dialog = AboutDialog()
        self.about_dialog.set_values(version)

        # 菜单栏设置
        self.setting_dialog = SettingDialog()
        self.setting_dialog.saved.connect(self.update_time_fmt)
        self.setting_dialog.saved.connect(self.update_lanzoucloud_settings)
        # 登录回收站信息更新器
        self.get_rec_lists_worker = GetRecListsWorker()
        self.get_rec_lists_worker.msg.connect(self.show_status)
        self.get_rec_lists_worker.infos.connect(self.update_rec_lists)
        self.get_rec_lists_worker.folders.connect(lambda: self.show_status('', 0))
        self.get_rec_lists_worker.folders.connect(self.pop_up_rec_folder_dialog)
        # 回收站操作器
        self.rec_manipulator = RecManipulator()
        self.rec_manipulator.msg.connect(self.show_status)
        self.rec_manipulator.succeeded.connect(self.get_rec_lists_worker.start)
        # 检查软件版本
        self.check_update_worker = CheckUpdateWorker()
        self.about_dialog.check_update.connect(self.check_update_worker.set_values)
        self.check_update_worker.infos.connect(self.about_dialog.show_update)
        self.check_update_worker.bg_update_infos.connect(self.show_new_version_msg)
        # 获取分享链接信息线程
        self.get_shared_info_thread = GetSharedInfo()
        # 验证码对话框
        # self.captcha_dialog = CaptchaDialog()
        # self.captcha_dialog.captcha.connect(self.set_captcha)
        # self.captcha_dialog.setWindowModality(Qt.WindowModality.ApplicationModal)

        self.merge_file_dialog = MergeFileDialog(USER_HOME)
        self.set_disk()

    # def set_captcha(self, code):
    #     self._captcha_code = code

    # def captcha_handler(self, img_data):
    #     """处理下载时出现的验证码(暂时不需要了)"""
    #     self.captcha_dialog.handle(img_data)
    #     self._captcha_code = None
    #     self.captcha_dialog.exec()
    #     from time import sleep
    #     while True:
    #         sleep(1)
    #         if self._captcha_code:
    #             break
    #     return self._captcha_code

    def show_toolbar_slot(self):
        if self.toolbar.isVisible():
            self.toolbar.close()
            self.show_toolbar.setText("显示工具栏")
        else:
            self.toolbar.show()
            self.toolbar.setIconSize(QSize(20, 20))
            self.show_toolbar.setText("关闭工具栏")

    def show_login_dialog(self):
        """显示登录对话框"""
        login_dialog = LoginDialog(self._config)
        login_dialog.clicked_ok.connect(self.call_login_launcher)
        login_dialog.setWindowModality(Qt.WindowModality.ApplicationModal)
        login_dialog.exec()

    def show_upload_dialog(self, files):
        """显示上传文件对话框"""
        if len(self._path_list) > 0:
            self.upload_dialog.set_values(self._path_list[-1].name, self._path_list[-1].id, files)
        else:
            self.show_status("等待文件列表更新...", 2000)

    def show_upload_dialog_menus(self):
        '''菜单栏显示上传对话框槽函数'''
        self.show_upload_dialog(None)

    def update_time_fmt(self):
        """更新文件界面的时间"""
        if self.tabWidget.currentIndex() == self.tabWidget.indexOf(self.disk_tab):
            self.show_file_and_folder_lists()

    def call_task_manager_thread(self, tasks: dict):
        self.task_manager.add_tasks(tasks)
        self._tasks.add(tasks)
        self.tabWidget.insertTab(3, self.jobs_tab, "任务管理")
        self.jobs_tab.setEnabled(True)
        self.show_jobs_lists()

    def call_multi_manipulator(self, action):
        """批量操作器"""
        tab_page = self.tabWidget.currentIndex()
        if tab_page == self.tabWidget.indexOf(self.share_tab):
            listview = self.table_share
            model = self.model_share
        elif tab_page == self.tabWidget.indexOf(self.disk_tab):
            listview = self.table_disk
            model = self.model_disk
        elif tab_page == self.tabWidget.indexOf(self.rec_tab):
            listview = self.table_rec
            model = self.model_rec
        else:
            return None
        infos = []
        _indexes = listview.selectionModel().selection().indexes()
        for i in _indexes:  # 获取所选行号
            info = model.item(i.row(), 0).data()
            if info and info not in infos:
                infos.append(info)

        if tab_page == self.tabWidget.indexOf(self.share_tab):  # 提取界面下载
            if not infos:
                return None
            tasks = {}
            first_item = infos[0]
            if first_item.all is None:  # 单文件信息链接 (info, None, 1, [])
                info = first_item.item  # ShareInfo(code=0, name, url, pwd, desc, time, size)
                tasks[info.url] = DlJob(infos=info, path=self._config.path, total_file=1)
            elif len(infos) != 1 and len(infos) >= first_item.count:  # 下载整个文件夹文件 (info, all, count, parent)
                info = first_item.all.folder  # 当前文件夹信息
                tasks[info.url] = DlJob(infos=info, path=self._config.path, total_file=first_item.count)
            else:  # 下载文件夹中部分文件
                parent_dir_lst = first_item.parent  # 父文件夹信息
                parent_dir = os.sep.join(parent_dir_lst)
                path = self._config.path + os.sep + parent_dir if parent_dir_lst else self._config.path
                for info in infos:
                    info = info.item  # 文件夹中单文件信息
                    tasks[info.url] = DlJob(infos=info, path=path, total_file=1)
            logger.debug(f"manipulator, share tab tasks={tasks}")
            self.call_task_manager_thread(tasks)
        elif tab_page == self.tabWidget.indexOf(self.disk_tab):  # 登录文件界面下载
            logger.error(f"登录文件界面下载{infos}")
            if not infos:
                return None
            logger.debug(f"manipulator, disk tab infos={infos}")
            self.desc_pwd_fetcher.set_values(infos, download=True, dl_path=self._config.path)
        elif tab_page == self.tabWidget.indexOf(self.rec_tab):  # 回收站
            logger.debug(f"manipulator, rec tab action={action}")
            title = msg = ""
            if action == "recovery":
                title = "确定恢复选定文件(夹)？"
            elif action == "delete":
                title = "确定彻底删除选定文件(夹)？"
            elif action == "recovery_all":
                title = "确定还原全部文件(夹)？"
                msg = "提示: 恢复回收站中的文件将不可撤销，请确认。"
            elif action == "clean":
                title = "确定清除全部文件(夹)？"
                msg = "提示: 删除回收站中的文件将不可恢复，请确认。"

            if action == "recovery" or action == "delete":
                if not infos:
                    self.show_status("请先选中需要操作的文件！", 2999)
                    return None
                msg = "\t\t列表：\n"
                for i in infos:
                    msg += f"{i.time}\t{i.name}\t{i.size}\n"
            message_box = QMessageBox(self)
            message_box.setIcon(QMessageBox.Icon.Warning)
            message_box.setStyleSheet(btn_style)
            message_box.setWindowTitle(title)
            message_box.setText(msg)
            message_box.setStandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No)
            buttonY = message_box.button(QMessageBox.StandardButton.Yes)
            buttonY.setText('确定')
            buttonN = message_box.button(QMessageBox.StandardButton.No)
            buttonN.setText('取消')
            message_box.exec()
            if message_box.clickedButton() == buttonY:
                self.rec_manipulator.set_values(infos, action)

    def call_logout_update_ui(self):
        """菜单栏、工具栏登出"""
        self.toolbar.removeAction(self.logout)
        self.tabWidget.setCurrentIndex(0)
        self.disk_tab.setEnabled(False)
        self.rec_tab.setEnabled(False)
        self.tabWidget.removeTab(2)
        self.tabWidget.removeTab(1)
        self.toolbar.removeAction(self.logout)  # 登出工具
        self.logout.setEnabled(False)
        self.toolbar.removeAction(self.upload)  # 上传文件工具栏
        self.upload.setEnabled(False)
        self._user = None

    def login_update_ui(self, success, msg, duration):
        """根据登录是否成功更新UI，并保持用户信息"""
        self.show_status(msg, duration)
        if success:
            self.update_lanzoucloud_settings()
            self._work_id = self._config.work_id  # 切换用户后刷新 根目录
            if self._user:
                if len(self._user) <= 6:
                    disk_tab = f"我的蓝奏<{self._user}>"
                else:
                    disk_tab = f"我的蓝奏<{self._user[:4]}..>"
            else:
                disk_tab = "我的蓝奏云"
            self.tabWidget.insertTab(1, self.disk_tab, disk_tab)
            self.tabWidget.insertTab(2, self.rec_tab, "回收站")
            self.disk_tab.setEnabled(True)
            self.rec_tab.setEnabled(True)
            # 更新快捷键与工具栏
            self.toolbar.addAction(self.logout)  # 添加登出工具栏
            self.toolbar.addAction(self.upload)  # 添加上传文件工具栏
            # 菜单栏槽
            self.logout.setEnabled(True)
            self.upload.setEnabled(True)
            self.download.setEnabled(True)
            self.delete.setEnabled(True)
            # 设置当前显示 tab
            self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.disk_tab))
            QCoreApplication.processEvents()  # 重绘界面
            self._config.update_user()  # 存储用户信息
            # 刷新文件列表
            # self.list_refresher.set_values(self._work_id)
        else:
            self.show_status(msg, duration)
            self.tabWidget.setCurrentIndex(0)
            self.tabWidget.removeTab(2)
            self.tabWidget.removeTab(1)
            self.disk_tab.setEnabled(False)
            self.rec_tab.setEnabled(False)
            # 更新快捷键与工具栏
            self.toolbar.removeAction(self.logout)  # 登出工具栏
            self.toolbar.removeAction(self.upload)  # 上传文件工具栏
            self.logout.setEnabled(False)
            self.upload.setEnabled(False)
            self.download.setEnabled(False)
            self.delete.setEnabled(False)

    def call_login_launcher(self):
        """登录网盘"""
        self.logout_worker.set_values(update_ui=False)
        self.toolbar.removeAction(self.logout)
        try:
            username = self._config.name
            password = self._config.pwd
            cookie = self._config.cookie
            if not username and not cookie:
                return None
            self.show_status("正在登陆，稍候……", 25000)
            self.login_launcher.set_values(username, password, cookie)
        except Exception as err:
            logger.error(f"Login: err={err}")

    def call_logout(self):
        """登出确认对话框"""
        message_box = QMessageBox(self)
        message_box.setStyleSheet(btn_style)
        message_box.setIcon(QMessageBox.Icon.Question)
        message_box.setWindowTitle("确认登出")
        message_box.setText("提示：登出不会删除已经保存的用户信息！\n\n是否确认登出？")
        message_box.setStandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No)
        buttonY = message_box.button(QMessageBox.StandardButton.Yes)
        buttonY.setText('确定')
        buttonN = message_box.button(QMessageBox.StandardButton.No)
        buttonN.setText('取消')
        message_box.accepted.connect(lambda: self.logout_worker.set_values(True))
        message_box.exec()

    def show_file_and_folder_lists(self):
        """显示用户文件和文件夹列表"""
        self.model_disk.removeRows(0, self.model_disk.rowCount())  # 清理旧的内容
        file_count = len(self._file_list)
        folder_count = len(self._folder_list)
        name_header = [f"文件夹{folder_count}个"] if folder_count else []
        if file_count:
            name_header.append(f"文件{file_count}个")
        self.model_disk.setHorizontalHeaderLabels(["/".join(name_header), "大小", "时间"])
        folder_ico = QIcon(SRC_DIR + "folder.gif")
        desc_style = ' <span style="font-size:14px;color:green;text-align:right">'
        pwd_ico = f' <img src="{SRC_DIR}keys.png" width="14" height="14" />'
        dl_count_style = ' <span style="font-size:14px;color:red;text-align:right">'
        if self._work_id != -1:
            _back = QStandardItem(folder_ico, "..")
            _back.setToolTip("双击返回上层文件夹，选中无效")
            self.model_disk.appendRow([_back, QStandardItem(""), QStandardItem("")])
        for infos in self._folder_list.values():  # 文件夹
            tips = ""
            name = QStandardItem()
            name.setData(FolderInfos(infos))
            name.setIcon(folder_ico)
            txt = infos.name + desc_style + infos.desc + "</span>" if infos.desc else infos.name
            if infos.has_pwd:
                tips = "有提取码"
                txt = txt + pwd_ico
                if infos.desc:
                    tips = tips + "，描述：" + str(infos.desc)
            elif infos.desc:
                tips = "描述：" + str(infos.desc)
            name.setText(txt)
            name.setToolTip(tips)
            self.model_disk.appendRow([name, QStandardItem(""), QStandardItem("")])
        for infos in self._file_list.values():  # 文件
            tips = ""
            name = QStandardItem(set_file_icon(infos.name), infos.name)
            name.setData(FileInfos(infos))
            txt = infos.name + desc_style + "有描述" + "</span>" if infos.has_des else infos.name
            if infos.has_pwd:
                tips = "有提取码"
                txt = txt + pwd_ico
                if infos.has_des:
                    tips = tips + "，有描述"
            elif infos.has_des:
                tips = "有描述"
            if infos.downs:
                txt = txt + dl_count_style + str(infos.downs) + "</span>"
            name.setText(txt)
            name.setToolTip(tips)
            size = QStandardItem(infos.size)
            size.setData(format_size_int(infos.size), Qt.ItemDataRole.UserRole)  # 配合MyStandardItem实现正确排序
            self.model_disk.appendRow([name, size, time_table_item(infos)])
        for row in range(self.model_disk.rowCount()):  # 右对齐
            self.model_disk.item(row, 1).setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
            self.model_disk.item(row, 2).setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)

    def update_disk_lists(self, infos):
        """更新用户文件列表"""
        if not infos:
            return
        if infos['r']['files']:
            self._file_list = infos['file_list']
        if infos['r']['folders']:
            self._folder_list = infos['folder_list']
            self._path_list = infos['path_list']

        self._work_id = self._path_list[-1].id
        self._config.work_id = self._work_id
        if len(self._path_list) > 1:
            self._parent_id = self._path_list[-2].id
        self.show_file_and_folder_lists()
        if infos['r']['path']:
            self.show_full_path()

    def config_tableview(self, tab):
        """Tab 初始化"""
        if tab == "share":
            model = self.model_share
            table = self.table_share
        elif tab == "disk":
            model = self.model_disk
            table = self.table_disk
        elif tab == "rec":
            model = self.model_rec
            table = self.table_rec
        elif tab == "jobs":
            model = self.model_jobs
            table = self.table_jobs
            table.doubleClicked.connect(self.open_downloaded_file)
        else:
            logger.error(f"Gui config_tableview: tab={tab}")
            return None

        if tab == "jobs":
            model.setHorizontalHeaderLabels(["任务名", "完成度", "状态", "操作"])
            table.horizontalHeader().resizeSection(3, 40)
        else:
            model.setHorizontalHeaderLabels(["文件名", "大小", "时间"])
        table.setItemDelegateForColumn(0, TableDelegate())  # table 支持富文本
        table.setModel(model)
        # 是否显示网格线
        table.setShowGrid(False)
        # 禁止编辑表格
        table.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        # 隐藏水平表头
        table.verticalHeader().setVisible(False)
        # 设置表头可以自动排序
        if tab == 'jobs':
            table.setSortingEnabled(False)
        else:
            table.setSortingEnabled(True)
        table.setMouseTracking(False)
        # 设置表头的背景色为绿色
        table.horizontalHeader().setStyleSheet("QHeaderView::section{background:lightgray}")
        # 设置 不可选择单个单元格，只可选择一行。
        table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        # 设置第二三列的宽度
        table.horizontalHeader().resizeSection(1, 76)
        table.horizontalHeader().resizeSection(2, 90)
        # 设置第一列宽度自动调整，充满屏幕
        table.horizontalHeader().setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)

        if tab != "rec" and tab != "jobs":
            table.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)  # 允许右键产生子菜单
            table.customContextMenuRequested.connect(self.generateMenu)  # 右键菜单

    def open_downloaded_file(self, index):
        data = self.model_jobs.itemData(index)
        for item in data.values():
            if not isinstance(item, DlJob):
                continue
            for task in self._tasks.values():
                if not isinstance(task, DlJob):
                    continue
                logger.error("open_downloaded_file state %s %s %s %s %s %s %s " % (task.type == 'dl',
                                                                                   task.name == item.name,
                                                                                   task.path == item.path,
                                                                                   task.is_finished(),
                                                                                   task.rate,
                                                                                   task.current,
                                                                                   task.total_size)
                             )
                if task.type == 'dl' and task.name == item.name and task.path == item.path and task.is_finished():
                    open_file(task)
                    return

    def call_rename_mkdir_worker(self, infos):
        """重命名、修改简介与新建文件夹"""
        self.rename_mkdir_worker.set_values(infos, self._work_id, self._folder_list)

    def set_passwd(self, infos):
        """设置文件(夹)提取码"""
        self.set_pwd_worker.set_values(infos, self._work_id)

    def on_moved(self, r_files=True, r_folders=True, r_path=True):
        """移动文件(夹)后更新界面槽函数"""
        self.list_refresher.set_values(self._work_id, r_files, r_folders, r_path)

    def call_mkdir(self):
        """弹出新建文件夹对话框"""
        self.rename_dialog.set_values()
        self.rename_dialog.exec()

    def call_remove_files(self):
        indexes = []
        infos = []
        _indexes = self.table_disk.selectionModel().selection().indexes()
        if not _indexes:
            return
        for i in _indexes:  # 获取所选行号
            indexes.append(i.row())
        indexes = set(indexes)
        for index in indexes:
            info = self.model_disk.item(index, 0).data()  # 用于提示删除的文件名
            if info:
                infos.append(info)
        delete_dialog = DeleteDialog(infos)
        delete_dialog.new_infos.connect(self.remove_files_worker.set_values)
        delete_dialog.exec()

    def call_copy_share_url(self, infos):
        self.pause_extract_clipboard(True)
        text = infos.url + ' 提取码：' + infos.pwd if infos.has_pwd else infos.url
        self.clipboard.setText(text)
        QTimer.singleShot(500, self.pause_extract_clipboard)
        self.show_status("已复制到剪切板", 2999)

    def generateMenu(self, pos):
        """右键菜单"""
        row_nums = self.sender().selectionModel().selection().indexes()
        if not row_nums:  # 如果没有选中行，什么也不做
            return
        _model = self.sender().model()
        infos = []  # 多个选中的行，用于移动文件与...
        for one_row in row_nums:
            row_data = _model.item(one_row.row(), 0).data()
            if row_data and row_data not in infos:  # 删掉 .. 行
                if isinstance(row_data, FileInfos):
                    infos.append(row_data)
                else:
                    infos.append(row_data)
        if not infos:
            return
        info = infos[0]  # 取选中的第一行
        # 通过是否有文件 ID 判断是登录界面还是提取界面
        if isinstance(info, (FileInfos, FolderInfos)):
            self.left_menu_rename_set_desc.setEnabled(True)
            self.left_menu_set_pwd.setEnabled(True)
            self.left_menu_move.setEnabled(True)
            # 文件夹不能移动，设置不同的显示菜单名
            if isinstance(info, FileInfos):
                self.left_menu_rename_set_desc.setText("修改文件描述（支持批量）")
            else:
                self.left_menu_rename_set_desc.setText("修改文件夹名与描述")
            if info.has_pwd:
                self.left_menu_copy.setText("复制分享链接与提取码")
            else:
                self.left_menu_copy.setText("复制分享链接")
        else:
            self.left_menu_rename_set_desc.setDisabled(True)
            self.left_menu_move.setDisabled(True)
            self.left_menu_set_pwd.setDisabled(True)

        action = self.left_menus.exec(self.sender().mapToGlobal(pos))
        if action == self.left_menu_share_url:  # 显示详细信息
            # 后台跟新信息，并显示信息对话框
            if isinstance(info, ShareItem):
                info = ShareFileInfos(info.item)  # 提取界面 info 在第一个位置
            self.more_info_worker.set_values(info)
            self.info_dialog.exec()
        elif action == self.left_menu_move:  # 移动文件
            self.all_folders_worker.set_values(infos)
        elif action == self.left_menu_set_pwd:  # 修改提取码
            if len(infos) == 1:
                self.desc_pwd_fetcher.set_values([info, ])  # 兼容下载器，使用列表的列表
            self.set_pwd_dialog.set_values(infos)
            self.set_pwd_dialog.exec()
        elif action == self.left_menu_rename_set_desc:  # 重命名与修改描述
            if len(infos) == 1:
                self.desc_pwd_fetcher.set_values([info, ])  # 兼容下载器，使用列表的列表
            self.rename_dialog.set_values(infos)
            self.rename_dialog.exec()
        elif action == self.left_menu_copy:  # 复制分享链接
            if isinstance(info, tuple) and len(info) == 3:
                info = ShareFileInfos(info[0])  # 提取界面 info 在第一个位置
            self.more_info_worker.set_values(info, emit_link=True)

    def call_update_desc_pwd(self, infos):
        """更新 desc、pwd"""
        self.rename_dialog.set_values(infos)
        self.set_pwd_dialog.set_values(infos)

    def call_change_dir(self, folder_id=-1):
        """顶部路径按钮调用"""

        def callfunc():
            self.list_refresher.set_values(folder_id)

        return callfunc

    def change_disk_dir(self, dir_name):
        """双击切换工作目录"""
        item = self.model_disk.item(dir_name.row(), 0)
        if item.text() == "..":  # 返回上级路径
            self.list_refresher.set_values(self._parent_id)
            return None
        dir_name = item.data().name  # 文件夹名
        if dir_name in self._folder_list.keys():
            folder_id = self._folder_list[dir_name].id
            self.list_refresher.set_values(folder_id)
        else:
            self.call_multi_manipulator("download")

    def call_upload(self, tasks: dict):
        """上传文件(夹)"""
        self._old_work_id = self._work_id  # 记录上传文件夹id
        self.task_manager.add_tasks(tasks)
        self.tabWidget.insertTab(3, self.jobs_tab, "任务管理")
        self.jobs_tab.setEnabled(True)
        self._tasks.add(tasks)

    def show_full_path(self):
        """路径框显示当前路径"""
        index = 1
        for _ in iter(self._path_list_old):
            self._locs[index].clicked.disconnect()
            self.disk_loc_hbox.removeWidget(self._locs[index])
            self._locs[index].deleteLater()
            self._locs[index] = None
            del self._locs[index]
            index += 1
        index = 1
        for item in iter(self._path_list):
            self._locs[index] = QPushButton(item.name, self.disk_tab)
            tip = f"fid:{item.id} | 描述:{item.desc}" if item.desc else f"fid:{item.id}"
            self._locs[index].setToolTip(tip)
            self._locs[index].setIcon(QIcon(SRC_DIR + "folder.gif"))
            self._locs[index].setStyleSheet("QPushButton {border:none; background:transparent;\
                                            color: rgb(139,0,139); font-weight:bold;}")
            self.disk_loc_hbox.insertWidget(index, self._locs[index])
            self._locs[index].clicked.connect(self.call_change_dir(item.id))
            index += 1
        self._path_list_old = self._path_list

    def select_all_btn(self, action="reverse"):
        """默认反转按钮状态"""
        page = self.tabWidget.currentIndex()
        if page == self.tabWidget.indexOf(self.share_tab):
            btn = self.btn_share_select_all
            table = self.table_share
        elif page == self.tabWidget.indexOf(self.disk_tab):
            btn = self.btn_disk_select_all
            table = self.table_disk
        elif page == self.tabWidget.indexOf(self.rec_tab):
            btn = self.btn_rec_select_all
            table = self.table_rec
        else:
            return
        if btn.isEnabled():
            if action == "reverse":
                if btn.text() == "全选":
                    table.selectAll()
                    btn.setText("取消")
                    btn.setIcon(QIcon(SRC_DIR + "select_none.png"))
                elif btn.text() == "取消":
                    table.clearSelection()
                    btn.setText("全选")
                    btn.setIcon(QIcon(SRC_DIR + "select_all.png"))
            elif action == "cancel":  # 点击列表其中一个就表示放弃全选
                btn.setText("全选")
                btn.setIcon(QIcon(SRC_DIR + "select_all.png"))
            else:
                table.selectAll()
                btn.setText("取消")
                btn.setIcon(QIcon(SRC_DIR + "select_none.png"))

    def call_delete_shortcut(self):
        if self.tabWidget.currentIndex() == self.tabWidget.indexOf(self.disk_tab):
            self.call_remove_files()

    def call_download_shortcut(self):
        if self.tabWidget.currentIndex() == self.tabWidget.indexOf(self.disk_tab):
            self.call_multi_manipulator("download")

    # disk tab
    def init_disk_ui(self):
        self.model_disk = QStandardItemModel(1, 3)
        self.config_tableview("disk")
        self.btn_disk_delete.setIcon(QIcon(SRC_DIR + "delete.png"))
        self.btn_disk_delete.setToolTip("按下 Ctrl + D 删除选中文件")
        self.btn_disk_dl.setIcon(QIcon(SRC_DIR + "downloader.png"))
        self.btn_disk_dl.setToolTip("按下 Ctrl + J 下载选中文件")
        self.btn_disk_select_all.setIcon(QIcon(SRC_DIR + "select_all.png"))
        self.btn_disk_select_all.setToolTip("按下 Ctrl/Alt + A 全选或则取消全选")
        self.btn_disk_select_all.clicked.connect(lambda: self.select_all_btn("reverse"))
        self.table_disk.clicked.connect(lambda: self.select_all_btn("cancel"))
        self.btn_disk_dl.clicked.connect(lambda: self.call_multi_manipulator("download"))
        self.btn_disk_mkdir.setIcon(QIcon(SRC_DIR + "add_folder.png"))
        self.btn_disk_mkdir.clicked.connect(self.call_mkdir)
        self.btn_disk_delete.clicked.connect(self.call_remove_files)
        # 文件拖拽上传
        self.table_disk.drop_files.connect(self.show_upload_dialog)
        self.table_disk.doubleClicked.connect(self.change_disk_dir)

    # rec tab
    def pop_up_rec_folder_dialog(self, files):
        # 弹出回收站文件夹内容对话框
        if files:
            rec_file_dialog = RecFolderDialog(files)
            rec_file_dialog.exec()
        else:
            self.show_status("文件夹为空！", 2999)

    def call_rec_folder_dialog(self, dir_name):
        # 显示弹出对话框
        dir_data = self.model_rec.item(dir_name.row(), 0).data()  # 文件夹信息
        if isinstance(dir_data, RecFolder):
            self.show_status(f"正在获取文件夹 {dir_data.name} 信息，稍候", 10000)
            self.get_rec_lists_worker.set_values(dir_data.id)

    def update_rec_lists(self, dir_lists, file_lists):
        """显示回收站文件和文件夹列表"""
        self.model_rec.removeRows(0, self.model_rec.rowCount())  # 清理旧的内容
        file_count = len(file_lists)
        folder_count = len(dir_lists)
        if (not dir_lists and not file_lists) or (file_count == 0 and folder_count == 0):
            self.show_status("回收站为空！", 4000)
            return
        name_header = ["文件夹{}个".format(folder_count), ] if folder_count else []
        if file_count:
            name_header.append("文件{}个".format(file_count))
        self.model_rec.setHorizontalHeaderLabels(["/".join(name_header), "大小", "时间"])
        folder_ico = QIcon(SRC_DIR + "folder.gif")

        for item in iter(dir_lists):  # 文件夹
            name = QStandardItem(folder_ico, item.name)
            name.setData(item)
            name.setToolTip("双击查看详情")
            size_ = QStandardItem(item.size)
            print("rec time_table_item dir")
            time_ = time_table_item(item)
            # time_ = QStandardItem(item.time)
            self.model_rec.appendRow([name, size_, time_])
        for item in iter(file_lists):  # 文件
            name = QStandardItem(set_file_icon(item.name), item.name)
            name.setData(item)
            size_ = QStandardItem(item.size)
            print("rec time_table_item")
            time_ = time_table_item(item)
            # time_ = QStandardItem(item.time)
            self.model_rec.appendRow([name, size_, time_])
        for row in range(self.model_rec.rowCount()):  # 右对齐
            self.model_rec.item(row, 1).setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
            self.model_rec.item(row, 2).setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)

    def init_rec_ui(self):
        """回收站ui初始化"""
        self.model_rec = QStandardItemModel(1, 3)
        self.config_tableview("rec")
        self.table_rec.doubleClicked.connect(self.call_rec_folder_dialog)
        self.btn_rec_select_all.setIcon(QIcon(SRC_DIR + "select_all.png"))
        self.btn_rec_select_all.clicked.connect(lambda: self.select_all_btn("reverse"))
        self.btn_rec_delete.clicked.connect(lambda: self.call_multi_manipulator("delete"))
        self.btn_rec_delete.setIcon(QIcon(SRC_DIR + "delete.png"))
        self.btn_recovery.clicked.connect(lambda: self.call_multi_manipulator("recovery"))
        self.btn_recovery.setIcon(QIcon(SRC_DIR + "rec_folder.png"))
        self.btn_rec_delete.setToolTip("彻底删除选中文件(夹)")
        self.btn_recovery.setToolTip("恢复选中文件(夹)")
        self.btn_recovery_all.clicked.connect(lambda: self.call_multi_manipulator("recovery_all"))
        self.btn_recovery_all.setIcon(QIcon(SRC_DIR + "rec_folder.png"))
        self.btn_recovery_all.setToolTip("恢复全部")
        self.btn_rec_clean.clicked.connect(lambda: self.call_multi_manipulator("clean"))
        self.btn_rec_clean.setIcon(QIcon(SRC_DIR + "rec_bin.png"))
        self.btn_rec_clean.setToolTip("清理回收站全部")
        self.btn_rec_expire_files.setToolTip("暂时无效！")

    # shared url
    def call_get_shared_info(self):
        """提取分享链接槽函数"""
        if not self.get_shared_info_thread.isRunning():  # 防止快速多次调用
            self._extract_show_dir = []
            self._extract_folder_list = {}
            self._extract_setted_head = False
            self._extract_count = 0
            self.line_share_url.setEnabled(False)
            self.line_share_pwd.setEnabled(False)
            self.btn_extract.setEnabled(False)
            text = self.line_share_url.text().strip()
            input_pwd = self.line_share_pwd.text().strip()
            self.model_share.setHorizontalHeaderLabels(['文件(夹)', "大小", "时间"])
            self.get_shared_info_thread.set_values(text, input_pwd)

    def show_share_folder_url_lists(self, infos, root_dir=[], show_dir=[]):
        # infos: 文件夹 FolderDetail -> code, folder, files, sub_folders
        folder_ico = QIcon(SRC_DIR + "folder.gif")
        if not (show_dir or self._extract_setted_head):
            self._extract_setted_head = True
            all_file = infos.folder.count  # 包括子文件夹的所有文件
            file_count = len(infos.files)  # 父文件夹中的文件数量
            folder_count = len(infos.sub_folders) or 0
            if self._show_subfolder:
                self._extract_count = all_file + folder_count  # 标记是否为全选下载
            else:
                self._extract_count = file_count + folder_count  # 标记是否为全选下载
            dots = '...' if len(infos.folder.desc) > 60 else ''
            desc = "| " + infos.folder.desc.replace('\n', ' ')[:60] + dots if infos.folder.desc else ""
            title = f"{infos.folder.name} | 文件{file_count}个 "
            if folder_count:
                title += f"| 子文件夹{folder_count}个 "
            if self._show_subfolder and all_file != file_count:
                title += f"| 递归文件{all_file}个 "
            title += f"｜ 总共 {infos.folder.size} {desc}"
            self.model_share.setHorizontalHeaderLabels([title, "大小", "时间"])

        if len(show_dir) == 1:
            _back = QStandardItem(folder_ico, "..")
            _back.setToolTip("双击返回上层文件夹，选中无效")
            _back.setData(ShareItem(all=infos))
            self.model_share.appendRow([_back, QStandardItem(""), QStandardItem("")])
        for sub_folder in iter(infos.sub_folders):  # 展示子文件夹
            if sub_folder.folder.name:  # 没有密码的子文件夹不展示在提取界面
                name = QStandardItem(folder_ico, sub_folder.folder.name)
                if root_dir:
                    root_path = [*show_dir, infos.folder.url]
                    post_root_dir = [*root_dir, sub_folder.folder.name]
                    pre_root_dir = f'<span style="font-size:14px;color:pink;text-align:right">{"/".join(root_dir)}/</span>'
                else:
                    root_path = [infos.folder.url, ]
                    pre_root_dir = ''
                    post_root_dir = [sub_folder.folder.name, ]
                if sub_folder.folder.desc:
                    text = ' <span style="font-size:14px;color:green;text-align:right">{}</span>'.format(
                        sub_folder.folder.desc.replace("\n", " "))
                else:
                    text = ''
                set_data = ShareItem(sub_folder.folder, infos, self._extract_count, root_dir)
                if not show_dir:
                    name.setData(set_data)
                    name.setText(pre_root_dir + sub_folder.folder.name + text)
                    if self.time_fmt:
                        time = QStandardItem(time_format(sub_folder.folder.time))
                    else:
                        time = time_table_item(sub_folder.folder)

                    size = QStandardItem(sub_folder.folder.size)
                    size.setData(format_size_int(sub_folder.folder.size), Qt.ItemDataRole.UserRole)
                    self.model_share.appendRow([name, size, time])
                if self._show_subfolder or show_dir:  # 在当前窗口递归展示子文件夹中的文件
                    self.show_share_folder_url_lists(sub_folder, root_dir=post_root_dir, show_dir=show_dir[1:])
                else:  # 不递归展示才会记录子文件夹信息
                    self._extract_folder_list[sub_folder.folder.url] = root_path
        if show_dir:
            return None
        for item in iter(infos.files):  # 展示文件
            if item:
                if root_dir:
                    pre_root_dir = f'<span style="font-size:14px;color:pink;text-align:right">{"/".join(root_dir)}/</span>'
                else:
                    pre_root_dir = ''
                name = QStandardItem(set_file_icon(item.name), item.name)
                name.setData(ShareItem(item, infos, self._extract_count, root_dir))
                name.setText(pre_root_dir + item.name)
                size = QStandardItem(item.size)
                size.setData(format_size_int(item.size), Qt.ItemDataRole.UserRole)
                # time = QStandardItem(time_format(item.time)) if self.time_fmt else QStandardItem(item.time)
                if self.time_fmt:
                    time = QStandardItem(time_format(item.time))
                else:
                    time = time_table_item(item)
                self.model_share.appendRow([name, size, time])

    def show_share_url_file_lists(self, infos):
        if infos.code == LanZouCloud.SUCCESS:
            if isinstance(infos, FolderDetail):  # 文件夹 FolderDetail -> code, folder, files, sub_folders
                self.show_share_folder_url_lists(infos)
            else:  # 单文件
                name = QStandardItem(set_file_icon(infos.name), infos.name)
                name.setData(ShareItem(item=infos))
                print("show url dir")
                item = time_table_item(infos)
                self.model_share.appendRow([name, QStandardItem(infos.size), item])
                self.model_share.setHorizontalHeaderLabels(["文件名", "大小", "时间"])
            for r in range(self.model_share.rowCount()):  # 右对齐
                self.model_share.item(r, 1).setTextAlignment(
                    Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
                self.model_share.item(r, 2).setTextAlignment(
                    Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
            self.table_share.setDisabled(False)
            self.btn_share_select_all.setDisabled(False)
            self.btn_share_select_all.setToolTip("按下 Ctrl/Alt + A 全选或则取消全选")
            self.btn_share_dl.setDisabled(False)
        else:
            self.btn_share_select_all.setDisabled(True)
            self.btn_share_select_all.setToolTip("")
            self.btn_share_dl.setDisabled(True)
            self.table_share.setDisabled(True)

    def set_download_path(self):
        """设置下载路径"""
        dl_path = QFileDialog.getExistingDirectory(self, "选择文件下载保存文件夹", self._config.path)
        dl_path = os.path.normpath(dl_path)  # windows backslash
        if dl_path == self._config.path or dl_path == ".":
            return None
        elif dl_path == "":
            self._config.default_path()
        else:
            self._config.path = dl_path
        self.share_set_dl_path.setText(self._config.path)

    def change_share_dir(self, dir_name):
        """双击子文件夹改变路径"""
        all_infos = self.model_share.item(dir_name.row(), 0).data()  #
        if self.model_share.item(dir_name.row(), 0).text() == "..":  # 返回父文件夹
            if self._extract_show_dir:
                self.model_share.removeRows(0, self.model_share.rowCount())
                self.show_share_folder_url_lists(all_infos.all, show_dir=self._extract_show_dir[:-1])
        elif all_infos.item.url in self._extract_folder_list.keys():
            show_url = self._extract_folder_list[all_infos.item.url]
            self._extract_show_dir = show_url
            self.model_share.removeRows(0, self.model_share.rowCount())
            self.show_share_folder_url_lists(all_infos.all, show_dir=show_url)

    def call_show_share_url_sub_folder(self, infos):
        """切换展示子文件夹按钮"""
        if self._show_subfolder:
            self._show_subfolder = False
            self._share_url_show_subfolder.setText("递归展示")
            self._share_url_show_subfolder.setToolTip("将子文件递归展示")
        else:
            self._show_subfolder = True
            self._extract_folder_list = {}  # 递归展示时不记录子文件夹信息
            self._share_url_show_subfolder.setText("收起文件")
            self._share_url_show_subfolder.setToolTip("将子文件收起，不展示")
        self.model_share.removeRows(0, self.model_share.rowCount())
        self.show_share_folder_url_lists(infos)

    def show_share_url_judge_folder(self, infos):
        """判断是否包含子文件夹"""
        try:
            if infos.sub_folders:
                logger.debug('Have subfolders!')
                self._share_url_show_subfolder = QPushButton("递归展示", self.share_tab)
                self._share_url_show_subfolder.setToolTip("将子文件递归展示")
                self._share_url_show_subfolder.setIcon(QIcon(SRC_DIR + "folder.gif"))
                self.share_hlayout_top.addWidget(self._share_url_show_subfolder)
                self._share_url_show_subfolder.clicked.connect(lambda: self.call_show_share_url_sub_folder(infos))
            else:
                logger.debug('Have No subfolders!')
                self.share_hlayout_top.removeWidget(self._share_url_show_subfolder)
                self._share_url_show_subfolder.deleteLater()
                self._share_url_show_subfolder = None
                del self._share_url_show_subfolder
        except:
            pass

    def init_extract_share_ui(self):
        self.btn_share_select_all.setDisabled(True)
        self.btn_share_dl.setDisabled(True)
        self.table_share.setDisabled(True)
        self.model_share = QStandardItemModel(1, 3)
        self.config_tableview("share")

        # 清理旧的信息
        self.get_shared_info_thread.clean.connect(lambda: self.model_share.removeRows(0, self.model_share.rowCount()))
        self.get_shared_info_thread.msg.connect(self.show_status)  # 提示信息
        self.get_shared_info_thread.infos.connect(self.show_share_url_judge_folder)  # 判断是否包含子文件夹
        self.get_shared_info_thread.infos.connect(self.show_share_url_file_lists)  # 内容信息
        self.get_shared_info_thread.update.connect(lambda: self.btn_extract.setEnabled(True))
        self.get_shared_info_thread.update.connect(lambda: self.line_share_url.setEnabled(True))
        self.get_shared_info_thread.update.connect(lambda: self.line_share_pwd.setEnabled(True))
        self.table_share.doubleClicked.connect(self.change_share_dir)  # 双击
        # 控件设置
        self.line_share_url.setPlaceholderText(
            "蓝奏云链接，如有提取码，放后面，空格或汉字等分割(汉字、特殊符号提取码手动右侧输入！)，回车键提取")
        self.line_share_url.returnPressed.connect(self.call_get_shared_info)
        self.line_share_url.setFocus()  # 光标焦点
        self.line_share_pwd.setPlaceholderText("特殊提取码")
        self.line_share_pwd.setFixedWidth(100)  # 提取码输入框 宽度
        self.btn_extract.clicked.connect(self.call_get_shared_info)
        self.btn_share_dl.clicked.connect(lambda: self.call_multi_manipulator("download"))
        self.btn_share_dl.setIcon(QIcon(SRC_DIR + "downloader.png"))
        self.btn_share_select_all.setIcon(QIcon(SRC_DIR + "select_all.png"))
        self.btn_share_select_all.clicked.connect(lambda: self.select_all_btn("reverse"))
        self.table_share.clicked.connect(lambda: self.select_all_btn("cancel"))  # 全选按钮

        self.extract_input_shortcut = QShortcut(QKeySequence("Ctrl+F"), self)
        self.extract_input_shortcut.activated.connect(self.call_extract_input_shortcut)

        # 添加文件下载路径选择器
        self.share_set_dl_path.setText(self._config.path)
        self.share_set_dl_path.clicked.connect(self.set_download_path)

    def call_extract_input_shortcut(self):
        """Ctrl+F槽函数"""
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.share_tab))
        self.line_share_url.selectAll()  # 全选中
        self.line_share_url.setFocus()  # 焦点

    # jobs tab
    def call_jobs_clean_all(self):
        self._tasks.clear()
        self.show_jobs_lists()

    def call_jobs_start_all(self):
        for task in self._tasks.values():
            if task.pause:
                task.pause = False
        self.task_manager.start()
        self.show_jobs_lists()

    def call_show_mgr_finished(self, num: int):
        if num:
            self.show_status(f"所有任务已完成！ <font color='red'>{num} 个任务失败</font>", 2999)
        else:
            self.show_status("所有任务已完成！", 2999)
        # 上传完成调用
        if self._old_work_id == self._work_id:
            self.list_refresher.set_values(self._work_id, True, True, False)
        else:
            self._old_work_id = self._work_id
        self.show_jobs_lists()

    def update_jobs_info(self):
        self._tasks.update()
        self.show_jobs_lists()

    def redo_job(self, task):
        logger.debug(f"re do job task={task}")
        self.task_manager.add_task(task)

    def start_work_job(self, task):
        self.task_manager.start_task(task)

    def stop_work_job(self, task):
        self.task_manager.stop_task(task)

    def del_work_job(self, task):
        self.task_manager.del_task(task)
        self._tasks.clear(task)
        self.show_jobs_lists()

    def show_jobs_lists(self):
        """显示任务列表"""
        self.model_jobs.removeRows(0, self.model_jobs.rowCount())  # 清理旧的内容
        download_ico = QIcon(SRC_DIR + "download.png")
        upload_ico = QIcon(SRC_DIR + "upload.png")
        path_style = ' <span style="font-size:14px;color:green;text-align:right">'
        error_style = ' <span style="font-size:14px;color:red;text-align:right">'
        _index = 0
        for task in self._tasks.values():
            name = QStandardItem()
            if task.type == 'dl':
                name.setIcon(download_ico)
                txt = task.name + '  ' + path_style + task.size + '|' + task.speed + '|' + task.prog \
                      + ' ➩ ' + task.path + "</span>"
            else:
                name.setIcon(upload_ico)
                txt = str(task.url[-100:]) + '  ' + path_style + change_size_unit(task.total_size) + '|' \
                      + task.speed + '|' + task.prog + ' ➩ ' + str(task.folder) + "</span>"
            if task.info:
                txt = txt + error_style + str(task.info) + "</span>"
            name.setText(txt)
            name.setToolTip(txt)
            name.setData(task)
            rate = "{:5.1f}%".format(task.rate / 10)
            percent = QStandardItem(rate)  # percent
            self.model_jobs.appendRow([name, percent, QStandardItem(""), QStandardItem("")])

            _status = QPushButton()
            _status.resize(_status.sizeHint())
            _action = QPushButton()
            _action.resize(_action.sizeHint())

            # _status.setDisabled(True)
            if task.is_finished():
                if task.type == "dl":
                    _status.setText("打开文件夹")
                    _status.clicked.connect(lambda: open_in_explorer(task))
                else:
                    _status.setText("已完成")
                _status.setStyleSheet(jobs_btn_completed_style)
                _action.setText("移除")
                _action.clicked.connect(lambda: self.del_work_job(task))
                _action.setStyleSheet(jobs_btn_delete_style)
            elif task.info:
                _status.setText("出错了")
                _status.setStyleSheet(jobs_btn_redo_style)
                if task.run:
                    _action.setText("暂停")
                    _action.clicked.connect(lambda: self.stop_work_job(task))
                else:
                    _action.setText("重试")
                    _action.clicked.connect(lambda: self.redo_job(task))
                _action.setStyleSheet(jobs_btn_completed_style)
            else:
                if task.run:
                    if task.type == 'dl':
                        _status.setText("下载中")
                    else:
                        _status.setText("上传中")
                    _status.setStyleSheet(jobs_btn_processing_style)
                    _action.setText("暂停")
                    _action.clicked.connect(lambda: self.stop_work_job(task))
                    _action.setStyleSheet(jobs_btn_completed_style)
                else:
                    if task.added:
                        _status.setText("排队中")
                        _status.setStyleSheet(jobs_btn_queue_style)

                        _action.setText("移除")
                        _action.clicked.connect(lambda: self.del_work_job(task))
                    else:
                        _status.setText("暂停中")
                        _status.setStyleSheet(jobs_btn_redo_style)

                        _action.setText("开始")
                        _action.clicked.connect(lambda: self.start_work_job(task))
                _action.setStyleSheet(jobs_btn_completed_style)

            self.table_jobs.setIndexWidget(self.model_jobs.index(_index, 2), _status)
            self.table_jobs.setIndexWidget(self.model_jobs.index(_index, 3), _action)
            _index += 1

        for row in range(self.model_jobs.rowCount()):  # 右对齐
            self.model_jobs.item(row, 1).setTextAlignment(Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignVCenter)
            self.model_jobs.item(row, 2).setTextAlignment(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)

    def init_jobs_ui(self):
        """初始化上传下载任务管理界面"""
        self.model_jobs = QStandardItemModel(1, 4)
        self.config_tableview("jobs")
        self.jobs_tab.setEnabled(True)
        self.btn_jobs_clean_all.clicked.connect(self.call_jobs_clean_all)  # 信号
        self.btn_jobs_start_all.clicked.connect(self.call_jobs_start_all)  # 信号

    # others
    def clean_status(self):
        self.statusbar_msg_label.setText("")
        self.statusbar_load_lb.clear()
        self.statusbar_load_movie.stop()

    def show_status(self, msg, duration=0):
        self.statusbar_msg_label.setText(msg)
        if msg and duration >= 3000:
            self.statusbar_load_lb.clear()
            self.statusbar_load_movie.stop()
            ht = self.statusbar_msg_label.size().height()
            self.statusbar_load_movie.setScaledSize(QSize(ht, ht))
            self.statusbar_load_lb.setMovie(self.statusbar_load_movie)
            self.statusbar_load_movie.start()
        else:
            self.statusbar_load_lb.clear()
            self.statusbar_load_movie.stop()
        if duration != 0:
            QTimer.singleShot(duration, self.clean_status)

    def call_change_tab(self):
        """切换标签页 动作"""
        tab_index = self.tabWidget.currentIndex()
        if tab_index == self.tabWidget.indexOf(self.share_tab):  # share 界面
            if self.get_shared_info_thread.isRunning():
                self.show_status("正在获取文件夹链接信息，可能需要几秒钟，请稍候……", 500000)
        elif tab_index == self.tabWidget.indexOf(self.rec_tab):  # rec 界面
            if not self.statusbar_msg_label.text():
                self.show_status("正在更新回收站...", 10000)
            self.get_rec_lists_worker.start()
        elif tab_index == self.tabWidget.indexOf(self.disk_tab):  # disk 界面
            if not self.statusbar_msg_label.text():
                self.show_status("正在更新当前目录...", 10000)
            self.list_refresher.set_values(self._work_id)
        elif tab_index == self.tabWidget.indexOf(self.jobs_tab):  # jobs 界面
            self.show_jobs_lists()

    def keyPressEvent(self, e):
        if e.key() == Qt.Key.Key_A:  # Ctrl/Alt + A 全选
            if e.modifiers() and Qt.ControlModifier:
                self.select_all_btn()
        elif e.key() == Qt.Key.Key_F5:  # 刷新
            self.call_change_tab()

    def open_wiki_url(self):
        # 打开使用说明页面
        url = QUrl('https://github.com/rachpt/lanzou-gui/wiki')
        if not QDesktopServices.openUrl(url):
            self.show_status('Could not open wiki page!', 5000)

    def show_new_version_msg(self, ver, msg):
        message_box = QMessageBox(self)
        message_box.setIcon(QMessageBox.Icon.Information)
        message_box.setStyleSheet(btn_style)
        message_box.setWindowTitle(f"检测到新版 {ver}")
        message_box.setText(msg)
        message_box.setStandardButtons(QMessageBox.StandardButton.Close)
        buttonC = message_box.button(QMessageBox.StandardButton.Close)
        buttonC.setText('关闭')
        message_box.exec()

    def closeEvent(self, event):
        if self._to_tray:
            event.ignore()
            self.hide()
            if self._show_to_tray_msg:
                self.tray.showMessage(
                    "已经最小化到托盘",
                    "双击显示/隐藏窗口，退出请右击",
                    QSystemTrayIcon.MessageIcon.Information,
                    3000
                )
                self._show_to_tray_msg = False  # 提示一次

    def Exit(self):
        # 点击关闭按钮或者点击退出事件会出现图标无法消失的bug，那就先隐藏吧(｡･ω･｡)
        self.tray.hide()
        del self.tray
        sys.exit(0)

    def pause_extract_clipboard(self, show=False):
        """登录文件界面屏蔽剪切板监听功能"""
        if show:
            self._watch_clipboard_old = self.watch_clipboard
            self.watch_clipboard = False
        else:
            try:
                self.watch_clipboard = self._watch_clipboard_old
            except:
                pass

    def auto_extract_clipboard(self):
        if not self.watch_clipboard:
            return
        text = self.clipboard.text()
        pat = r"(https?://(\w[-\w]*\.)?lanzou[a-z].com/[a-z]?[-/a-zA-Z0-9]+)[^a-zA-Z0-9]*([a-zA-Z0-9]+)?"
        for share_url, _, pwd in re.findall(pat, text):
            if share_url and not self.get_shared_info_thread.isRunning():
                self.line_share_url.setEnabled(False)
                self.line_share_pwd.setEnabled(False)
                self.btn_extract.setEnabled(False)
                txt = share_url + "提取码：" + pwd if pwd else share_url
                self.line_share_url.setText(txt)
                self.get_shared_info_thread.set_values(txt)
                self.tabWidget.setCurrentIndex(0)
                self.setWindowFlags(Qt.WindowType.FramelessWindowHint | Qt.WindowType.WindowStaysOnTopHint)  # 窗口最前
                self.show()
                self.setWindowFlags(Qt.WindowType.WindowCloseButtonHint | Qt.WindowType.WindowMinMaxButtonsHint)  # 窗口恢复
                self.show()
                break

    def clipboard_monitor(self):
        """监听系统剪切板"""
        self.clipboard = QApplication.clipboard()
        self.clipboard.dataChanged.connect(self.auto_extract_clipboard)



from PyQt6.QtCore import QThread, pyqtSignal

from lanzou.api import why_error
from lanzou.api.utils import is_folder_url, is_file_url
from lanzou.debug import logger


class Downloader(QThread):
    """单个文件下载线程"""
    finished_ = pyqtSignal(object)
    folder_file_failed = pyqtSignal(object, object)
    failed = pyqtSignal()
    proc = pyqtSignal(str)
    update = pyqtSignal()

    def __init__(self, disk, task, callback, parent=None):
        super(Downloader, self).__init__(parent)
        self._disk = disk
        self._task = task
        self._callback_thread = callback(task)

    def stop(self):
        self.terminate()

    def _callback(self):
        """显示进度条的回调函数"""
        if not self._callback_thread.isRunning():
            self.update.emit()
            self.proc.emit(self._callback_thread.emit_msg)
            self._callback_thread.start()

    def _down_failed(self, code, file):
        """显示下载失败的回调函数"""
        self.folder_file_failed.emit(code, file)

    def __del__(self):
        self.wait()

    def run(self):
        if not self._task:
            logger.error("Download task is empty!")
            return None
        self._task.run = True
        try:
            if is_file_url(self._task.url):  # 下载文件
                res = self._disk.down_file_by_url(self._task.url, self._task, self._callback)
            elif is_folder_url(self._task.url):  # 下载文件夹
                res = self._disk.down_dir_by_url(self._task, self._callback)
            else:
                raise UserWarning
            if res == 0:
                self._task.rate = 1000  # 回调线程可能在休眠
                self.update.emit()
            else:
                self._task.info = why_error(res)
                logger.debug(f"Download : res={res}")
                self.failed.emit()
        except TimeoutError:
            self._task.info = "网络连接错误！"
            logger.error("Download TimeOut")
            self.failed.emit()
        except Exception as err:
            self._task.info = f"未知错误！err={err}"
            logger.error(f"Download error: err={err}")
            self.failed.emit()

        self._task.run = False
        self.finished_.emit(self._task)



from PyQt6 import QtCore, QtGui, QtWidgets

from lanzou.debug import SRC_DIR
from lanzou.gui.others import MyLineEdit, MyTableView


class Ui_MainWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super(Ui_MainWindow, self).__init__()

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setEnabled(True)
        MainWindow.resize(1000, 650)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.share_tab = QtWidgets.QWidget()
        self.share_tab.setObjectName("share_tab")
        self.share_vlayout = QtWidgets.QVBoxLayout(self.share_tab)
        self.share_vlayout.setObjectName("share_vlayout")
        self.share_hlayout_top = QtWidgets.QHBoxLayout()
        self.share_hlayout_top.setObjectName("share_hlayout_top")
        self.label_share_url = QtWidgets.QLabel(self.share_tab)
        self.label_share_url.setObjectName("label_share_url")
        self.share_hlayout_top.addWidget(self.label_share_url)
        self.line_share_url = QtWidgets.QLineEdit(self.share_tab)
        self.line_share_url.setObjectName("line_share_url")
        self.line_share_pwd = QtWidgets.QLineEdit(self.share_tab)
        self.line_share_pwd.setObjectName("line_share_pwd")
        self.share_hlayout_top.addWidget(self.line_share_url)
        self.share_hlayout_top.addWidget(self.line_share_pwd)
        self.btn_extract = QtWidgets.QPushButton(self.share_tab)
        self.btn_extract.setObjectName("btn_extract")
        self.share_hlayout_top.addWidget(self.btn_extract)
        self.share_vlayout.addLayout(self.share_hlayout_top)
        self.table_share = QtWidgets.QTableView(self.share_tab)
        self.table_share.setObjectName("table_share")
        self.share_vlayout.addWidget(self.table_share)
        self.share_hlayout_bottom = QtWidgets.QHBoxLayout()
        self.share_hlayout_bottom.setObjectName("share_hlayout_bottom")
        self.btn_share_select_all = QtWidgets.QPushButton(self.share_tab)
        self.btn_share_select_all.setObjectName("btn_share_select_all")
        self.share_hlayout_bottom.addWidget(self.btn_share_select_all)
        self.label_dl_path = QtWidgets.QLabel(self.share_tab)
        self.label_dl_path.setObjectName("label_dl_path")
        self.share_hlayout_bottom.addWidget(self.label_dl_path)
        self.share_set_dl_path = MyLineEdit(self.share_tab)
        self.share_set_dl_path.setObjectName("share_set_dl_path")
        self.share_hlayout_bottom.addWidget(self.share_set_dl_path)
        self.btn_share_dl = QtWidgets.QPushButton(self.share_tab)
        self.btn_share_dl.setObjectName("btn_share_dl")
        self.share_hlayout_bottom.addWidget(self.btn_share_dl)
        self.share_vlayout.addLayout(self.share_hlayout_bottom)
        self.tabWidget.addTab(self.share_tab, "")
        self.disk_tab = QtWidgets.QWidget()
        self.disk_tab.setEnabled(True)
        self.disk_tab.setMinimumSize(QtCore.QSize(620, 0))
        self.disk_tab.setObjectName("disk_tab")
        self.disk_vlayout = QtWidgets.QVBoxLayout(self.disk_tab)
        self.disk_vlayout.setObjectName("disk_vlayout")
        self.disk_hlayout_top = QtWidgets.QHBoxLayout()
        self.disk_hlayout_top.setObjectName("disk_hlayout_top")
        self.disk_loc_hbox = QtWidgets.QHBoxLayout()
        self.disk_loc_hbox.setObjectName("disk_loc_hbox")
        self.label_disk_loc = QtWidgets.QLabel(self.disk_tab)
        self.label_disk_loc.setObjectName("label_disk_loc")
        self.disk_loc_hbox.addWidget(self.label_disk_loc)
        self.disk_hlayout_top.addLayout(self.disk_loc_hbox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding,
                                           QtWidgets.QSizePolicy.Policy.Minimum)
        self.disk_hlayout_top.addItem(spacerItem)
        self.btn_disk_mkdir = QtWidgets.QPushButton(self.disk_tab)
        self.btn_disk_mkdir.setObjectName("btn_disk_mkdir")
        self.disk_hlayout_top.addWidget(self.btn_disk_mkdir)
        self.disk_vlayout.addLayout(self.disk_hlayout_top)
        self.table_disk = MyTableView(self.disk_tab)
        self.table_disk.setObjectName("table_disk")
        self.disk_vlayout.addWidget(self.table_disk)
        self.disk_hlayout_bottom = QtWidgets.QHBoxLayout()
        self.disk_hlayout_bottom.setObjectName("disk_hlayout_bottom")
        self.btn_disk_select_all = QtWidgets.QPushButton(self.disk_tab)
        self.btn_disk_select_all.setObjectName("btn_disk_select_all")
        self.disk_hlayout_bottom.addWidget(self.btn_disk_select_all)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding,
                                            QtWidgets.QSizePolicy.Policy.Minimum)
        self.disk_hlayout_bottom.addItem(spacerItem1)
        self.btn_disk_delete = QtWidgets.QPushButton(self.disk_tab)
        self.btn_disk_delete.setObjectName("btn_disk_delete")
        self.disk_hlayout_bottom.addWidget(self.btn_disk_delete)
        self.btn_disk_dl = QtWidgets.QPushButton(self.disk_tab)
        self.btn_disk_dl.setObjectName("btn_disk_dl")
        self.disk_hlayout_bottom.addWidget(self.btn_disk_dl)
        self.disk_vlayout.addLayout(self.disk_hlayout_bottom)
        self.tabWidget.addTab(self.disk_tab, "")

        self.rec_tab = QtWidgets.QWidget()
        self.rec_tab.setObjectName("rec_tab")
        self.rec_verticalLayout = QtWidgets.QVBoxLayout(self.rec_tab)
        self.rec_verticalLayout.setObjectName("rec_verticalLayout")
        self.rec_horizontalLayout = QtWidgets.QHBoxLayout()
        self.rec_horizontalLayout.setObjectName("rec_horizontalLayout")
        self.btn_rec_select_all = QtWidgets.QPushButton(self.rec_tab)
        self.btn_rec_select_all.setObjectName("btn_rec_select_all")
        self.rec_horizontalLayout.addWidget(self.btn_rec_select_all)
        self.btn_recovery = QtWidgets.QPushButton(self.rec_tab)
        self.btn_recovery.setObjectName("btn_recovery")
        self.rec_horizontalLayout.addWidget(self.btn_recovery)
        self.btn_rec_delete = QtWidgets.QPushButton(self.rec_tab)
        self.btn_rec_delete.setObjectName("btn_rec_delete")
        self.rec_horizontalLayout.addWidget(self.btn_rec_delete)
        self.rec_horizontalLayout.addStretch(1)
        self.btn_rec_clean = QtWidgets.QPushButton(self.rec_tab)
        self.btn_rec_clean.setObjectName("btn_rec_clean")
        self.rec_horizontalLayout.addWidget(self.btn_rec_clean)
        self.btn_recovery_all = QtWidgets.QPushButton(self.rec_tab)
        self.btn_recovery_all.setObjectName("btn_recovery_all")
        self.rec_horizontalLayout.addWidget(self.btn_recovery_all)
        self.btn_rec_expire_files = QtWidgets.QPushButton(self.rec_tab)
        self.btn_rec_expire_files.setObjectName("btn_rec_expire_files")
        self.rec_horizontalLayout.addWidget(self.btn_rec_expire_files)
        self.rec_verticalLayout.addLayout(self.rec_horizontalLayout)
        self.table_rec = QtWidgets.QTableView(self.rec_tab)
        self.table_rec.setObjectName("table_rec")
        self.rec_verticalLayout.addWidget(self.table_rec)
        self.tabWidget.addTab(self.rec_tab, "")

        self.jobs_tab = QtWidgets.QWidget()
        self.jobs_tab.setObjectName("jobs_tab")
        self.jobs_verticalLayout = QtWidgets.QVBoxLayout(self.jobs_tab)
        self.jobs_verticalLayout.setObjectName("jobs_verticalLayout")
        self.jobs_horizontalLayout = QtWidgets.QHBoxLayout()
        self.jobs_horizontalLayout.setObjectName("jobs_horizontalLayout")
        self.btn_jobs_start_all = QtWidgets.QPushButton(self.jobs_tab)
        self.btn_jobs_clean_all = QtWidgets.QPushButton(self.jobs_tab)
        self.jobs_horizontalLayout.addWidget(self.btn_jobs_start_all)
        self.jobs_horizontalLayout.addStretch(1)
        self.jobs_horizontalLayout.addWidget(self.btn_jobs_clean_all)
        self.table_jobs = MyTableView(self.jobs_tab)
        self.table_jobs.setObjectName("table_jobs")
        self.jobs_verticalLayout.addLayout(self.jobs_horizontalLayout)
        self.jobs_verticalLayout.addWidget(self.table_jobs)
        self.tabWidget.addTab(self.jobs_tab, "")

        self.verticalLayout.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 640, 30))
        self.menubar.setObjectName("menubar")
        self.acount = QtWidgets.QMenu(self.menubar)
        self.acount.setObjectName("acount")
        self.files = QtWidgets.QMenu(self.menubar)
        self.files.setObjectName("files")
        self.help = QtWidgets.QMenu(self.menubar)
        self.help.setObjectName("help")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolbar = QtWidgets.QToolBar(MainWindow)
        self.toolbar.setObjectName("toolbar")
        MainWindow.addToolBar(QtCore.Qt.ToolBarArea.TopToolBarArea, self.toolbar)
        self.login = QtGui.QAction(MainWindow)
        self.login.setObjectName("login")
        self.logout = QtGui.QAction(MainWindow)
        self.logout.setObjectName("logout")
        self.upload = QtGui.QAction(MainWindow)
        self.upload.setObjectName("upload")
        self.download = QtGui.QAction(MainWindow)
        self.download.setObjectName("download")
        self.delete = QtGui.QAction(MainWindow)
        self.delete.setObjectName("delete")
        self.show_toolbar = QtGui.QAction(MainWindow)
        self.show_toolbar.setObjectName("show_toolbar")
        self.merge_file = QtGui.QAction(MainWindow)
        self.merge_file.setObjectName("merge_file")
        self.setting_menu = QtGui.QAction(MainWindow)
        self.setting_menu.setObjectName("setting_menu")
        self.how = QtGui.QAction(MainWindow)
        self.how.setObjectName("how")
        self.about = QtGui.QAction(MainWindow)
        self.about.setObjectName("about")
        self.acount.addSeparator()
        self.acount.addAction(self.login)
        self.acount.addAction(self.logout)
        self.files.addAction(self.upload)
        self.files.addAction(self.download)
        self.files.addAction(self.delete)
        self.files.addAction(self.show_toolbar)
        self.files.addAction(self.merge_file)
        self.files.addAction(self.setting_menu)
        self.help.addAction(self.how)
        self.help.addAction(self.about)
        self.menubar.addAction(self.acount.menuAction())
        self.menubar.addAction(self.files.menuAction())
        self.menubar.addAction(self.help.menuAction())
        # 工具栏
        self.toolbar.addAction(self.login)
        # 状态栏
        self.statusbar_msg_label = QtWidgets.QLabel()
        self.statusbar_load_lb = QtWidgets.QLabel()
        self.statusbar_load_movie = QtGui.QMovie(SRC_DIR + "loading_more.gif")
        self.statusbar_load_lb.setMovie(self.statusbar_load_movie)
        self.statusbar_msg_label.setObjectName("msg_label")
        self.statusbar_load_lb.setObjectName("msg_movie_lb")
        self.statusbar.addWidget(self.statusbar_load_lb)
        self.statusbar.addWidget(self.statusbar_msg_label)

        self.retranslateUi(MainWindow)
        self.other_init()
        self.set_window_at_center()
        self.create_left_menus()
        self.init_main_menu()
        self.text_add_shadow()
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label_share_url.setText(_translate("MainWindow", "蓝奏链接"))
        self.btn_extract.setText(_translate("MainWindow", "提取"))
        self.btn_share_select_all.setText(_translate("MainWindow", "全选"))
        self.label_dl_path.setText(_translate("MainWindow", "文件下载路径"))
        self.btn_share_dl.setText(_translate("MainWindow", "下载选中文件"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.share_tab), _translate("MainWindow", "蓝奏云链接提取"))
        self.label_disk_loc.setText(_translate("MainWindow", "位置"))
        self.btn_disk_mkdir.setText(_translate("MainWindow", "新建文件夹"))
        self.btn_disk_select_all.setText(_translate("MainWindow", "全选"))
        self.btn_disk_delete.setText(_translate("MainWindow", "删除"))
        self.btn_disk_dl.setText(_translate("MainWindow", "下载"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.disk_tab), _translate("MainWindow", "我的蓝奏云"))
        self.btn_rec_select_all.setText(_translate("MainWindow", "全选"))
        self.btn_recovery.setText(_translate("MainWindow", "还原"))
        self.btn_rec_delete.setText(_translate("MainWindow", "彻底删除"))
        self.btn_rec_clean.setText(_translate("MainWindow", "清空回收站"))
        self.btn_recovery_all.setText(_translate("MainWindow", "全部还原"))
        self.btn_rec_expire_files.setText(_translate("MainWindow", "过期文件"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.rec_tab), _translate("MainWindow", "回收站"))
        self.btn_jobs_start_all.setText(_translate("MainWindow", "开始所有暂停任务"))
        self.btn_jobs_clean_all.setText(_translate("MainWindow", "删除已完成任务"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.jobs_tab), _translate("MainWindow", "任务管理"))
        self.acount.setTitle(_translate("MainWindow", "登录"))
        self.files.setTitle(_translate("MainWindow", "文件"))
        self.help.setTitle(_translate("MainWindow", "帮助"))
        self.toolbar.setWindowTitle(_translate("MainWindow", "工具栏"))
        self.login.setText(_translate("MainWindow", "登录"))
        self.logout.setText(_translate("MainWindow", "登出"))
        self.upload.setText(_translate("MainWindow", "上传"))
        self.download.setText(_translate("MainWindow", "下载"))
        self.delete.setText(_translate("MainWindow", "删除"))
        self.show_toolbar.setText(_translate("MainWindow", "显示工具栏"))
        self.merge_file.setText(_translate("MainWindow", "合并文件"))
        self.setting_menu.setText(_translate("MainWindow", "设置"))
        self.how.setText(_translate("MainWindow", "使用说明"))
        self.about.setText(_translate("MainWindow", "关于"))

    def other_init(self):
        # 设置 tab
        self.tabWidget.setCurrentIndex(0)
        self.tabWidget.removeTab(3)
        self.tabWidget.removeTab(2)
        self.tabWidget.removeTab(1)
        self.disk_tab.setEnabled(False)
        self.rec_tab.setEnabled(False)
        self.jobs_tab.setEnabled(False)

    def set_window_at_center(self):
        # screen = QtGui.QScreen().geometry()
        # screen = QtWidgets.QDesktopWidget().screenGeometry()
        # size = self.geometry()
        # new_left = int((screen.width() - size.width()) / 2)
        # new_top = int((screen.height() - size.height()) / 2)
        # self.move(new_left, new_top)
        pass

    def create_left_menus(self):
        self.left_menus = QtWidgets.QMenu()
        self.left_menu_share_url = self.left_menus.addAction("外链分享地址等")
        self.left_menu_share_url.setIcon(QtGui.QIcon(SRC_DIR + "share.png"))
        self.left_menu_rename_set_desc = self.left_menus.addAction("修改文件描述（支持批量）")
        self.left_menu_rename_set_desc.setIcon(QtGui.QIcon(SRC_DIR + "desc.png"))
        self.left_menu_set_pwd = self.left_menus.addAction("设置提取码（支持批量）")
        self.left_menu_set_pwd.setIcon(QtGui.QIcon(SRC_DIR + "password.png"))
        self.left_menu_move = self.left_menus.addAction("移动（支持批量）")
        self.left_menu_move.setIcon(QtGui.QIcon(SRC_DIR + "move.png"))
        self.left_menu_copy = self.left_menus.addAction("复制分享链接")
        self.left_menu_copy.setIcon(QtGui.QIcon(SRC_DIR + "count.png"))

    def init_main_menu(self):
        self.login.setIcon(QtGui.QIcon(SRC_DIR + "login.png"))
        self.login.setShortcut("Ctrl+L")
        self.logout.setIcon(QtGui.QIcon(SRC_DIR + "logout.png"))
        self.logout.setShortcut("Ctrl+Q")  # 登出快捷键
        self.logout.setEnabled(False)  # 初始状态
        self.download.setIcon(QtGui.QIcon(SRC_DIR + "download.png"))
        self.download.setShortcut("Ctrl+J")
        self.download.setEnabled(False)  # 初始状态
        self.delete.setIcon(QtGui.QIcon(SRC_DIR + "delete.png"))
        self.delete.setShortcut("Ctrl+D")
        self.delete.setEnabled(False)  # 初始状态
        self.show_toolbar.setIcon(QtGui.QIcon(SRC_DIR + "password.png"))
        self.show_toolbar.setShortcut("Ctrl+T")
        self.toolbar.close()  # 工具栏初始状态
        self.merge_file.setIcon(QtGui.QIcon(SRC_DIR + "folder.gif"))
        self.merge_file.setShortcut("Ctrl+M")
        self.setting_menu.setIcon(QtGui.QIcon(SRC_DIR + "settings.png"))
        self.setting_menu.setShortcut("Ctrl+P")
        self.how.setIcon(QtGui.QIcon(SRC_DIR + "help.png"))
        self.how.setShortcut("F1")
        self.about.setIcon(QtGui.QIcon(SRC_DIR + "about.png"))
        self.about.setShortcut("Ctrl+B")
        self.upload.setIcon(QtGui.QIcon(SRC_DIR + "upload.png"))
        self.upload.setShortcut("Ctrl+U")  # 上传快捷键
        self.upload.setEnabled(False)  # 初始状态

    def text_add_shadow(self):
        share_url_shadow = QtWidgets.QGraphicsDropShadowEffect()
        share_url_shadow.setBlurRadius(4)
        share_url_shadow.setColor(QtGui.QColor('red'))
        share_url_shadow.setOffset(0)
        self.label_share_url.setGraphicsEffect(share_url_shadow)

        dl_path_shadow = QtWidgets.QGraphicsDropShadowEffect()
        dl_path_shadow.setBlurRadius(4)
        dl_path_shadow.setColor(QtGui.QColor('green'))
        dl_path_shadow.setOffset(0)
        self.label_dl_path.setGraphicsEffect(dl_path_shadow)

        disk_loc_shadow = QtWidgets.QGraphicsDropShadowEffect()
        disk_loc_shadow.setBlurRadius(5)
        disk_loc_shadow.setColor(QtGui.QColor('white'))
        disk_loc_shadow.setOffset(0)
        self.label_disk_loc.setGraphicsEffect(disk_loc_shadow)



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger


class SetPwdWorker(QThread):
    '''设置文件(夹)提取码 线程'''
    msg = pyqtSignal(str, int)
    update = pyqtSignal(object, object, object, object)

    def __init__(self, parent=None):
        super(SetPwdWorker, self).__init__(parent)
        self._disk = None
        self.infos = []
        self._work_id = -1
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos, work_id):
        self.infos = infos
        self._work_id = work_id
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                has_file = False
                has_folder = False
                failed = False
                failed_code = ""
                for infos in self.infos:
                    if infos.is_file:  # 文件
                        has_file = True
                        new_pwd = infos.new_pwd
                        if 2 > len(new_pwd) >= 1 or len(new_pwd) > 6:
                            self.msg.emit("文件提取码为2-6位字符,关闭请留空！", 4000)
                            raise UserWarning
                    else:  # 文件夹
                        has_folder = True
                        new_pwd = infos.new_pwd
                        if 2 > len(new_pwd) >= 1 or len(new_pwd) > 12:
                            self.msg.emit("文件夹提取码为0-12位字符,关闭请留空！", 4000)
                            raise UserWarning
                    res = self._disk.set_passwd(infos.id, infos.new_pwd, infos.is_file)
                    if res != LanZouCloud.SUCCESS:
                        failed_code = failed_code + str(res)
                        failed = True
                if failed:
                    self.msg.emit(f"❌部分提取码变更失败:{failed_code}，请勿使用特殊符号!", 4000)
                else:
                    self.msg.emit("✅提取码变更成功！♬", 3000)

                self.update.emit(self._work_id, has_file, has_folder, False)
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except UserWarning:
                pass
            except Exception as e:
                logger.error(f"SetPwdWorker error: e={e}")
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



from time import sleep

from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger


class RenameMkdirWorker(QThread):
    """重命名、修改简介与新建文件夹 线程"""
    # infos = pyqtSignal(object, object)
    msg = pyqtSignal(str, int)
    update = pyqtSignal(object, object, object, object)

    def __init__(self, parent=None):
        super(RenameMkdirWorker, self).__init__(parent)
        self._disk = None
        self._work_id = -1
        self._folder_list = None
        self.infos = None
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos, work_id, folder_list):
        self.infos = infos  # 对话框标识文件与文件夹
        self._work_id = work_id
        self._folder_list = folder_list
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True

            action = self.infos[0]
            try:
                if action == 'new':  # 新建文件夹
                    new_name = self.infos[1]
                    new_des = self.infos[2]
                    if new_name in self._folder_list.keys():
                        self.msg.emit(f"文件夹已存在：{new_name}", 7000)
                    else:
                        res = self._disk.mkdir(self._work_id, new_name, new_des)
                        if res == LanZouCloud.MKDIR_ERROR:
                            self.msg.emit(f"创建文件夹失败：{new_name}", 7000)
                        else:
                            sleep(1.5)  # 暂停一下，否则无法获取新建的文件夹
                            self.update.emit(self._work_id, False, True, False)  # 此处仅更新文件夹，并显示
                            self.msg.emit(f"成功创建文件夹：{new_name}", 4000)
                else:  # 重命名、修改简介
                    has_file = False
                    has_folder = False
                    failed = False
                    for info in self.infos[1]:
                        if info.is_file:  # 修改文件描述
                            res = self._disk.set_desc(info.id, info.new_des, is_file=info.is_file)
                            if res == LanZouCloud.SUCCESS:
                                has_file = True
                            else:
                                failed = True
                        else:  # 修改文件夹，action == "folder"
                            name = info.new_name or info.nmae
                            res = self._disk._set_dir_info(info.id, str(name), str(info.new_des))
                            if res == LanZouCloud.SUCCESS:
                                has_folder = True
                            else:
                                failed = True
                    self.update.emit(self._work_id, has_file, has_folder, False)
                    if failed:
                        self.msg.emit("有发生错误！", 6000)
                    else:
                        self.msg.emit("修改成功！", 4000)
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except Exception as e:
                logger.error(f"RenameMikdirWorker error: e={e}")

            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



import re

import requests
from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.debug import logger


class CheckUpdateWorker(QThread):
    '''检测软件更新'''
    infos = pyqtSignal(object, object)
    bg_update_infos = pyqtSignal(object, object)

    def __init__(self, parent=None):
        super(CheckUpdateWorker, self).__init__(parent)
        self._ver = ''
        self._manual = False
        self._mutex = QMutex()
        self._is_work = False
        self._folder_id = None
        self._api = 'https://api.github.com/repos/Leon406/lanzou-gui/releases/latest'
        self._api_mirror = 'https://gitee.com/api/v5/repos/rachpt/lanzou-gui/releases/latest'

    def set_values(self, ver: str, manual: bool = False):
        # 检查更新
        self._ver = ver
        self._manual = manual
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            resp = None
            try:
                resp = requests.get(self._api).json()
            except (requests.RequestException, TimeoutError, requests.exceptions.ConnectionError):
                logger.debug("chcek update from github error")
                try:
                    resp = requests.get(self._api_mirror).json()
                except:
                    logger.debug("chcek update from gitee error")
            except Exception as e:
                logger.error(f"CheckUpdateWorker error: e={e}")
            if resp:
                try:
                    tag_name, msg = resp['tag_name'], resp['body']
                    ver = self._ver.replace('v', '').split('-')[0].split('.')
                    ver2 = tag_name.replace('v', '').split('-')[0].split('.')
                    local_version = int(ver[0]) * 100 + int(ver[1]) * 10 + int(ver[2])
                    remote_version = int(ver2[0]) * 100 + int(ver2[1]) * 10 + int(ver2[2])
                    if remote_version > local_version:
                        urls = re.findall(r'https?://[-\.a-zA-Z0-9/_#?&%@]+', msg)
                        for url in urls:
                            new_url = f'<a href="{url}">{url}</a>'
                            msg = msg.replace(url, new_url)
                        msg = msg.replace('\n', '<br />')
                        self.infos.emit(tag_name, msg)
                        if not self._manual:  # 打开软件时检测更新
                            self.bg_update_infos.emit(tag_name, msg)
                    elif self._manual:
                        self.infos.emit("0", "目前还没有发布新版本！")
                except AttributeError:
                    if self._manual:
                        self.infos.emit("v0.0.0", "检查更新时发生异常，请重试！")
                except Exception as e:
                    logger.error(f"Check Update Version error: e={e}")
            else:
                if self._manual:
                    self.infos.emit("v0.0.0",
                                    f"检查更新时 <a href='{self._api}'>api.github.com</a>、<a href='{self._api_mirror}'>gitee.com</a> 拒绝连接，请稍后重试！")
            self._manual = False
            self._is_work = False
            self._mutex.unlock()
        else:
            if self._manual:
                self.infos.emit("v0.0.0", "后台正在运行，请稍等！")



"""
容器类，用于储存 上传、下载 任务，文件、文件夹信息
"""

import os


class Job:
    def __init__(self, url, tp, total_file=1):
        self._url = url
        self._type = tp
        self._total_file = total_file
        self._err_info = None
        self._run = False
        self._rate = 0
        self._current = 1
        self._speed = ''
        self._pause = False
        self._added = False
        self._now_size = 0
        self._total_size = 0

    @property
    def url(self):
        return self._url

    @property
    def info(self):
        return self._err_info

    @info.setter
    def info(self, info):
        self._err_info = info

    @property
    def run(self):
        return self._run

    @run.setter
    def run(self, run):
        self._run = run

    @property
    def rate(self):
        return self._rate

    @rate.setter
    def rate(self, rate):
        self._rate = rate

    @property
    def total_file(self):
        return self._total_file

    @total_file.setter
    def total_file(self, total_file):
        self._total_file = total_file

    @property
    def current(self):
        return self._current

    @current.setter
    def current(self, current):
        self._current = current

    @property
    def speed(self):
        return self._speed

    @speed.setter
    def speed(self, speed):
        self._speed = speed

    @property
    def prog(self):
        return f"({self._current}/{self._total_file})" if self._total_file > 1 else ''

    @property
    def type(self):
        return self._type

    @property
    def now_size(self):
        return self._now_size

    @now_size.setter
    def now_size(self, now_size):
        self._now_size = now_size

    @property
    def total_size(self):
        return self._total_size

    @total_size.setter
    def total_size(self, total_size):
        self._total_size = total_size

    @property
    def pause(self):
        return self._pause

    @pause.setter
    def pause(self, pause):
        self._pause = pause

    @property
    def added(self):
        return self._added

    @added.setter
    def added(self, added):
        self._added = added

    def is_finished(self) -> bool:
        return self.rate >= 1000 and (self.current >= self.total_file or self.current == 1 and self.total_file != 0)


class DlJob(Job):
    def __init__(self, infos, path, total_file=1):
        """info: lanzou.gui.models.FileInfos | ShareInfo
        ShareInfo(code=0, name, url, pwd, desc, time, size)
        """
        super(DlJob, self).__init__(infos.url, 'dl', total_file)
        self._infos = infos
        self._path = path
        self.size = infos.size

    @property
    def name(self):
        return self._infos.name

    @property
    def pwd(self):
        return self._infos.pwd

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path):
        self._path = path

    def __str__(self):
        return f"DlJob: name={self.name}, pwd={self.pwd} path= {self.path}"


class UpJob(Job):
    def __init__(self, url, fid, folder, pwd=None, desc=None, total_size=0, total_file=1):
        super(UpJob, self).__init__(url, 'up')
        self._fid = fid
        self._folder = folder
        self._pwd = pwd
        self._desc = desc
        self._total_size = total_size
        self._total_file = total_file

    @property
    def fid(self):
        return self._fid

    @property
    def name(self):
        return os.path.basename(self._url)

    @property
    def folder(self):
        return self._folder

    @property
    def pwd(self):
        return self._pwd

    @property
    def desc(self):
        return self._desc

    def __str__(self):
        return f"UpJob: fid={self.fid}, folder={self.folder},name={self.name}, pwd={self.pwd} desc= {self.desc}"


class Tasks(object):
    def __init__(self):
        self._items = {}
        self._dones = {}
        self._all = {}

    def __len__(self):
        return len(self._all)

    def __getitem__(self, index):
        return self._all[index]

    def __iter__(self):
        return iter(self._items)

    def add(self, tasks):
        """添加任务"""
        for key, value in tasks.items():
            if value.rate >= 1000:
                self._dones.update({key: value})
                if key in self._items:
                    del self._items[key]
            else:
                self._items.update({key: value})
                if key in self._dones:
                    del self._dones[key]
        self._all = {**self._items, **self._dones}

    def update(self):
        """更新元素"""
        changed = False
        for key, value in self._items.copy().items():
            if value.rate >= 1000:
                self._dones.update({key: value})
                del self._items[key]
                changed = True
        if changed:
            self._all = {**self._items, **self._dones}

    def items(self):
        return self._all.items()

    def values(self):
        return self._all.values()

    def clear(self, task=None):
        """清空元素"""
        if task:
            if task.url in self._dones:
                del self._dones[task.url]
            elif task.url in self._items:
                del self._items[task.url]
        else:
            self._dones.clear()
        self._all = {**self._items, **self._dones}


class Infos:
    def __init__(self, name='', is_file=True, fid='', time='', size='', downs=0, desc='', pwd='', url='', durl=''):
        self._name = name
        self._is_file = is_file
        self._fid = fid
        self._time = time
        self._size = size
        self._downs = downs
        self._desc = desc
        self._pwd = pwd
        self._url = url
        self._durl = durl
        self._has_pwd = False
        self._new_pwd = ''
        self._new_des = ''
        self._new_name = ''
        self._new_fid = ''

    @property
    def name(self):
        return self._name

    @property
    def is_file(self):
        return self._is_file

    @is_file.setter
    def is_file(self, is_file):
        self._is_file = is_file

    @property
    def id(self):
        return self._fid

    @property
    def size(self):
        return self._size

    @property
    def time(self):
        return self._time

    @property
    def downs(self):
        return self._downs

    @property
    def desc(self):
        return self._desc

    @desc.setter
    def desc(self, desc):
        self._desc = desc

    @property
    def pwd(self):
        return self._pwd

    @pwd.setter
    def pwd(self, pwd):
        self._pwd = pwd

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, url):
        self._url = url

    @property
    def durl(self):
        return self._durl

    @durl.setter
    def durl(self, durl):
        self._durl = durl

    @property
    def has_pwd(self):
        return self._has_pwd

    @property
    def new_pwd(self):
        return self._new_pwd

    @new_pwd.setter
    def new_pwd(self, new_pwd):
        self._new_pwd = new_pwd

    @property
    def new_des(self):
        return self._new_des

    @new_des.setter
    def new_des(self, new_des):
        self._new_des = new_des

    @property
    def new_name(self):
        return self._new_name

    @new_name.setter
    def new_name(self, new_name):
        self._new_name = new_name

    @property
    def new_id(self):
        return self._new_fid

    @new_id.setter
    def new_id(self, new_id):
        self._new_fid = new_id

    def __str__(self):
        return f"Infos: name={self._name}, is_file={self._is_file}, url={self._url}, size={self._size}, " \
               f"time={self._time}, durl={self._durl}"


class FileInfos(Infos):
    def __init__(self, file):
        super(FileInfos, self).__init__(is_file=True)
        self._name = file.name
        self._fid = file.id
        self._time = file.time
        self._size = file.size
        self._downs = file.downs
        self._has_pwd = file.has_pwd
        self._has_des = file.has_des

    @property
    def has_des(self):
        return self._has_des


class FolderInfos(Infos):
    def __init__(self, folder):
        super(FolderInfos, self).__init__(is_file=False)
        self._name = folder.name
        self._fid = folder.id
        self._desc = folder.desc
        self._has_pwd = folder.has_pwd


class ShareFileInfos(Infos):
    def __init__(self, file):
        super(ShareFileInfos, self).__init__(is_file=True)
        self._name = file.name
        self._time = file.time
        self._size = file.size
        self._url = file.url
        self._pwd = file.pwd



"""
重新封装的控件
"""

import os

from PyQt6.QtCore import Qt, pyqtSignal, QTimer, QSize
from PyQt6.QtGui import QTextDocument, QAbstractTextDocumentLayout, QPalette, QFontMetrics, QIcon, QStandardItem
from PyQt6.QtWidgets import (QApplication, QAbstractItemView, QStyle, QListView, QLineEdit, QTableView,
                             QPushButton, QStyledItemDelegate, QStyleOptionViewItem, QTextEdit, QSizePolicy)

from lanzou.debug import SRC_DIR


def set_file_icon(name):
    suffix = name.split(".")[-1]
    ico_path = SRC_DIR + f"{suffix}.gif"
    if os.path.isfile(ico_path):
        return QIcon(ico_path)
    else:
        return QIcon(SRC_DIR + "file.png")


class QDoublePushButton(QPushButton):
    """加入了双击事件的按钮"""
    doubleClicked = pyqtSignal()
    clicked = pyqtSignal()

    def __init__(self, *args, **kwargs):
        QPushButton.__init__(self, *args, **kwargs)
        self.timer = QTimer()
        self.timer.setSingleShot(True)
        self.timer.timeout.connect(self.clicked.emit)
        super().clicked.connect(self.checkDoubleClick)

    def checkDoubleClick(self):
        if self.timer.isActive():
            self.doubleClicked.emit()
            self.timer.stop()
        else:
            self.timer.start(250)


class MyLineEdit(QLineEdit):
    """添加单击事件的输入框，用于设置下载路径"""

    clicked = pyqtSignal()

    def __init__(self, parent):
        super(MyLineEdit, self).__init__(parent)

    def mouseReleaseEvent(self, QMouseEvent):
        if QMouseEvent.button() == Qt.MouseButton.LeftButton:
            self.clicked.emit()


class MyListView(QListView):
    """加入拖拽功能的列表显示器"""
    drop_files = pyqtSignal(object)

    def __init__(self):
        QListView.__init__(self)

        self.setDragDropMode(QAbstractItemView.DragDropMode.InternalMove)
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)

    def dragEnterEvent(self, event):
        m = event.mimeData()
        if m.hasUrls():
            for url in m.urls():
                if url.isLocalFile():
                    event.accept()
                    return
        event.ignore()

    def dropEvent(self, event):
        if event.source():
            QListView.dropEvent(self, event)
        else:
            m = event.mimeData()
            if m.hasUrls():
                urls = [url.toLocalFile() for url in m.urls() if url.isLocalFile()]
                if urls:
                    self.drop_files.emit(urls)
                    event.acceptProposedAction()


class AutoResizingTextEdit(QTextEdit):
    """添加单击事件的自动改变大小的文本输入框，用于显示描述与下载直链
    https://github.com/cameel/auto-resizing-text-edit
    https://gist.github.com/hahastudio/4345418
    """
    clicked = pyqtSignal()
    editingFinished = pyqtSignal()

    def __init__(self, parent=None):
        super(AutoResizingTextEdit, self).__init__(parent)

        # This seems to have no effect. I have expected that it will cause self.hasHeightForWidth()
        # to start returning True, but it hasn't - that's why I hardcoded it to True there anyway.
        # I still set it to True in size policy just in case - for consistency.
        size_policy = self.sizePolicy()
        size_policy.setHeightForWidth(True)
        size_policy.setVerticalPolicy(QSizePolicy.Policy.Preferred)
        self.setSizePolicy(size_policy)
        self.textChanged.connect(self.updateGeometry)

        self._changed = False
        self.setTabChangesFocus(True)
        self.textChanged.connect(self._handle_text_changed)

    def setMinimumLines(self, num_lines):
        """ Sets minimum widget height to a value corresponding to specified number of lines
            in the default font. """
        pass
        self.setMinimumSize(self.minimumSize().width(), self.lineCountToWidgetHeight(num_lines))

    def heightForWidth(self, width):
        margins = self.contentsMargins()

        if width >= margins.left() + margins.right():
            document_width = width - margins.left() - margins.right()
        else:
            # If specified width can't even fit the margin, there's no space left for the document
            document_width = 0

        # Cloning the whole document only to check its size at different width seems wasteful
        # but apparently it's the only and preferred way to do this in Qt >= 4. QTextDocument does not
        # provide any means to get height for specified width (as some QWidget subclasses do).
        # Neither does QTextEdit. In Qt3 Q3TextEdit had working implementation of heightForWidth()
        # but it was allegedly just a hack and was removed.
        #
        # The performance probably won't be a problem here because the application is meant to
        # work with a lot of small notes rather than few big ones. And there's usually only one
        # editor that needs to be dynamically resized - the one having focus.
        document = self.document().clone()
        document.setTextWidth(document_width)

        return margins.top() + document.size().height() + margins.bottom()

    def sizeHint(self):
        original_hint = super(AutoResizingTextEdit, self).sizeHint()
        return QSize(original_hint.width(), self.heightForWidth(original_hint.width()))

    def mouseReleaseEvent(self, QMouseEvent):
        if QMouseEvent.button() == Qt.MouseButton.LeftButton:
            if not self.toPlainText():
                self.clicked.emit()

    def lineCountToWidgetHeight(self, num_lines):
        """ Returns the number of pixels corresponding to the height of specified number of lines
            in the default font. """

        # ASSUMPTION: The document uses only the default font

        assert num_lines >= 0

        widget_margins = self.contentsMargins()
        document_margin = self.document().documentMargin()
        font_metrics = QFontMetrics(self.document().defaultFont())

        # font_metrics.lineSpacing() is ignored because it seems to be already included in font_metrics.height()
        return int(
                widget_margins.top() +
                document_margin +
                max(num_lines, 1) * font_metrics.height() +
                self.document().documentMargin() +
                widget_margins.bottom()
        )

    def focusOutEvent(self, event):
        if self._changed:
            self.editingFinished.emit()
        super(AutoResizingTextEdit, self).focusOutEvent(event)

    def _handle_text_changed(self):
        self._changed = True


class TableDelegate(QStyledItemDelegate):
    """Table 富文本"""

    def __init__(self, parent=None):
        super(TableDelegate, self).__init__(parent)
        self.doc = QTextDocument(self)

    def paint(self, painter, option, index):
        painter.save()
        options = QStyleOptionViewItem(option)
        self.initStyleOption(options, index)
        self.doc.setHtml(options.text)
        options.text = ""  # 原字符
        style = QApplication.style() if options.widget is None else options.widget.style()
        style.drawControl(QStyle.ControlElement.CE_ItemViewItem, options, painter)

        ctx = QAbstractTextDocumentLayout.PaintContext()

        if option.state & QStyle.StateFlag.State_Selected:
            ctx.palette.setColor(QPalette.ColorRole.Text, option.palette.color(
                QPalette.ColorGroup.Active, QPalette.ColorRole.HighlightedText))
        else:
            ctx.palette.setColor(QPalette.ColorRole.Text, option.palette.color(
                QPalette.ColorGroup.Active, QPalette.ColorRole.Text))

        text_rect = style.subElementRect(QStyle.SubElement.SE_ItemViewItemText, options)

        the_fuck_your_shit_up_constant = 3  # ￣へ￣ #
        margin = (option.rect.height() - options.fontMetrics.height()) // 2
        margin = margin - the_fuck_your_shit_up_constant
        text_rect.setTop(text_rect.top() + margin)

        painter.translate(text_rect.topLeft())
        painter.setClipRect(text_rect.translated(-text_rect.topLeft()))
        self.doc.documentLayout().draw(painter, ctx)

        painter.restore()

    def sizeHint(self, option, index):
        options = QStyleOptionViewItem(option)
        self.initStyleOption(options, index)
        self.doc.setHtml(options.text)
        self.doc.setTextWidth(options.rect.width())
        return QSize(self.doc.idealWidth(), self.doc.size().height())


class MyStandardItem(QStandardItem):
    def __lt__(self, other):
        data = self.data(Qt.ItemDataRole.UserRole)
        other_data = other.data(Qt.ItemDataRole.UserRole)
        if data and other_data:
            return data < other_data
        else:  # 没有setData并设置UserRole，则使用默认的方式进行比较排序
            return self.text() < other.text()


class MyTableView(QTableView):
    """加入拖拽功能的表格显示器"""
    drop_files = pyqtSignal(object)

    def __init__(self, parent):
        super(MyTableView, self).__init__(parent)
        self.setDragDropMode(QAbstractItemView.DragDropMode.InternalMove)
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)

    def dragEnterEvent(self, event):
        m = event.mimeData()
        if m.hasUrls():
            for url in m.urls():
                if url.isLocalFile():
                    event.accept()
                    return
        event.ignore()

    def dropEvent(self, event):
        if event.source():
            QListView.dropEvent(self, event)
        else:
            m = event.mimeData()
            if m.hasUrls():
                urls = [url.toLocalFile() for url in m.urls() if url.isLocalFile()]
                if urls:
                    self.drop_files.emit(urls)
                    event.acceptProposedAction()



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger
from lanzou.gui.models import DlJob


class DescPwdFetcher(QThread):
    """获取描述与提取码 线程"""
    desc = pyqtSignal(object)
    tasks = pyqtSignal(object)
    msg = pyqtSignal(object, object)

    def __init__(self, parent=None):
        super(DescPwdFetcher, self).__init__(parent)
        self._disk = None
        self.infos = None
        self.download = False
        self.dl_path = ""
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos, download=False, dl_path=""):
        self.infos = infos  # 列表的列表
        self.download = download  # 标识激发下载器
        self.dl_path = dl_path
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                if not self.infos:
                    raise UserWarning
                _tasks = {}
                _infos = []
                for info in self.infos:
                    if info.id:  # disk 运行
                        if info.is_file:  # 文件
                            res = self._disk.get_share_info(info.id, is_file=True)
                        else:  # 文件夹
                            res = self._disk.get_share_info(info.id, is_file=False)
                        if res.code == LanZouCloud.SUCCESS:
                            info.pwd = res.pwd
                            info.url = res.url
                            info.desc = res.desc
                        elif res.code == LanZouCloud.NETWORK_ERROR:
                            self.msg.emit("网络错误，请稍后重试！", 6000)
                            continue
                    _infos.append(info)  # info -> lanzou.gui.models.FileInfos
                    _tasks[info.url] = DlJob(infos=info, path=self.dl_path, total_file=1)
                if self.download:
                    self.tasks.emit(_tasks)
                else:  # 激发简介更新
                    self.desc.emit(_infos)
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except UserWarning:
                pass
            except Exception as e:
                logger.error(f"GetPwdFetcher error: e={e}")
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行指令！请稍后重试", 3100)



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import QDialog, QLabel, QGridLayout, QDialogButtonBox, QLineEdit, QTextEdit

from lanzou.debug import SRC_DIR
from lanzou.gui.qss import dialog_qss_style


class RenameDialog(QDialog):
    out = pyqtSignal(object)

    def __init__(self, parent=None):
        super(RenameDialog, self).__init__(parent)
        self.infos = []
        self.min_width = 400
        self.initUI()
        self.update_text()
        self.setStyleSheet(dialog_qss_style)

    def set_values(self, infos=None):
        self.infos = infos or []
        self.update_text()  # 更新界面

    def initUI(self):
        self.setWindowIcon(QIcon(SRC_DIR + "desc.png"))
        self.lb_name = QLabel()
        self.lb_name.setText("文件夹名：")
        self.lb_name.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)
        self.tx_name = QLineEdit()
        self.lb_desc = QLabel()
        self.tx_desc = QTextEdit()
        self.lb_desc.setText("描　　述：")
        self.lb_desc.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")

        self.grid = QGridLayout()
        self.grid.setSpacing(10)
        self.grid.addWidget(self.lb_name, 1, 0)
        self.grid.addWidget(self.tx_name, 1, 1)
        self.grid.addWidget(self.lb_desc, 2, 0)
        self.grid.addWidget(self.tx_desc, 2, 1, 5, 1)
        self.grid.addWidget(self.buttonBox, 7, 1, 1, 1)
        self.setLayout(self.grid)
        self.buttonBox.accepted.connect(self.btn_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

    def update_text(self):
        self.tx_desc.setFocus()
        num = len(self.infos)
        if num == 1:
            self.lb_name.setVisible(True)
            self.tx_name.setVisible(True)
            infos = self.infos[0]
            self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setToolTip("")  # 去除新建文件夹影响
            self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(True)  # 去除新建文件夹影响
            self.setWindowTitle("修改文件夹名与描述")
            self.tx_name.setText(str(infos.name))
            if infos.desc:
                self.tx_desc.setText(str(infos.desc))
                self.tx_desc.setToolTip('原描述：' + str(infos.desc))
            else:
                self.tx_desc.setText("")
                self.tx_desc.setToolTip('')
            self.tx_desc.setPlaceholderText("无")
            self.min_width = len(str(infos.name)) * 8
            if infos.is_file:
                self.setWindowTitle("修改文件描述")
                self.tx_name.setFocusPolicy(Qt.FocusPolicy.NoFocus)
                self.tx_name.setReadOnly(True)
            else:
                self.tx_name.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
                self.tx_name.setReadOnly(False)
                self.tx_name.setFocus()
        elif num > 1:
            self.lb_name.setVisible(False)
            self.tx_name.setVisible(False)
            self.setWindowTitle(f"批量修改{num}个文件(夹)的描述")
            self.tx_desc.setText('')
            self.tx_desc.setPlaceholderText("建议160字数以内。")

        else:
            self.setWindowTitle("新建文件夹")
            self.tx_name.setText("")
            self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False)
            self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setToolTip("请先输入文件名！")
            self.tx_name.textChanged.connect(self.slot_new_ok_btn)
            self.tx_name.setPlaceholderText("不支持空格，如有会被自动替换成 _")
            self.tx_name.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
            self.tx_name.setReadOnly(False)
            self.tx_desc.setPlaceholderText("可选项，建议160字数以内。")
            self.tx_name.setFocus()
        if self.min_width < 400:
            self.min_width = 400
        self.resize(self.min_width, 200)

    def slot_new_ok_btn(self):
        """新建文件夹槽函数"""
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(True)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setToolTip("")

    def btn_ok(self):
        new_name = self.tx_name.text()
        new_des = self.tx_desc.toPlainText()
        info_len = len(self.infos)
        if info_len == 0:  # 在 work_id 新建文件夹
            if new_name:
                self.out.emit(("new", new_name, new_des))
        elif info_len == 1:
            if new_name != self.infos[0].name or new_des != self.infos[0].desc:
                self.infos[0].new_des = new_des
                self.infos[0].new_name = new_name
                self.out.emit(("change", self.infos))
        else:
            if new_des:
                for infos in self.infos:
                    infos.new_des = new_des
                self.out.emit(("change", self.infos))
